@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 from dataclasses import dataclass
-from typing import TYPE_CHECKING, Any, Optional, Union
+from typing import TYPE_CHECKING, Any
 
 from typing_extensions import Self
 
@@ -15,9 +15,6 @@ if TYPE_CHECKING:
     from infrahub.permissions import AssignedPermissions
 
 
-# pylint: disable=redefined-builtin
-
-
 @dataclass
 class GlobalPermission:
     action: str
@@ -60,7 +57,7 @@ class AccountGlobalPermissionQuery(Query):
         self.account_id = account_id
         super().__init__(**kwargs)
 
-    async def query_init(self, db: InfrahubDatabase, **kwargs: Any) -> None:
+    async def query_init(self, db: InfrahubDatabase, **kwargs: Any) -> None:  # noqa: ARG002
         self.params["account_id"] = self.account_id
 
         branch_filter, branch_params = self.branch.get_query_filter_path(
@@ -175,7 +172,7 @@ class AccountObjectPermissionQuery(Query):
         self.account_id = account_id
         super().__init__(**kwargs)
 
-    async def query_init(self, db: InfrahubDatabase, **kwargs: Any) -> None:
+    async def query_init(self, db: InfrahubDatabase, **kwargs: Any) -> None:  # noqa: ARG002
         self.params["account_id"] = self.account_id
 
         branch_filter, branch_params = self.branch.get_query_filter_path(
@@ -327,7 +324,7 @@ class AccountRoleGlobalPermissionQuery(Query):
         self.role_id = role_id
         super().__init__(**kwargs)
 
-    async def query_init(self, db: InfrahubDatabase, **kwargs: Any) -> None:
+    async def query_init(self, db: InfrahubDatabase, **kwargs: Any) -> None:  # noqa: ARG002
         self.params["role_id"] = self.role_id
 
         branch_filter, branch_params = self.branch.get_query_filter_path(
@@ -417,7 +414,7 @@ class AccountRoleObjectPermissionQuery(Query):
         self.role_id = role_id
         super().__init__(**kwargs)
 
-    async def query_init(self, db: InfrahubDatabase, **kwargs: Any) -> None:
+    async def query_init(self, db: InfrahubDatabase, **kwargs: Any) -> None:  # noqa: ARG002
         self.params["role_id"] = self.role_id
 
         branch_filter, branch_params = self.branch.get_query_filter_path(
@@ -544,7 +541,7 @@ class AccountTokenValidateQuery(Query):
         self.token = token
         super().__init__(**kwargs)
 
-    async def query_init(self, db: InfrahubDatabase, **kwargs: Any) -> None:
+    async def query_init(self, db: InfrahubDatabase, **kwargs: Any) -> None:  # noqa: ARG002
         token_filter_perms, token_params = self.branch.get_query_filter_relationships(
             rel_labels=["r1", "r2"], at=self.at, include_outside_parentheses=True
         )
@@ -573,14 +570,14 @@ class AccountTokenValidateQuery(Query):
 
         self.return_labels = ["at", "av", "acc"]
 
-    def get_account_name(self) -> Optional[str]:
+    def get_account_name(self) -> str | None:
         """Return the account name that matched the query or None."""
         if result := self.get_result():
             return result.get("av").get("value")
 
         return None
 
-    def get_account_id(self) -> Optional[str]:
+    def get_account_id(self) -> str | None:
         """Return the account id that matched the query or a None."""
         if result := self.get_result():
             return result.get("acc").get("uuid")
@@ -588,7 +585,7 @@ class AccountTokenValidateQuery(Query):
         return None
 
 
-async def validate_token(token: str, db: InfrahubDatabase, branch: Optional[Union[Branch, str]] = None) -> str | None:
+async def validate_token(token: str, db: InfrahubDatabase, branch: Branch | str | None = None) -> str | None:
     branch = await registry.get_branch(db=db, branch=branch)
     query = await AccountTokenValidateQuery.init(db=db, branch=branch, token=token)
     await query.execute(db=db)
