@@ -3,17 +3,18 @@ from __future__ import annotations
 import ipaddress
 import re
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Optional, Union
+from typing import TYPE_CHECKING, Any
 
 import netaddr
 import ujson
-from infrahub_sdk.timestamp import TimestampFormatError
+from infrahub_sdk.exceptions import TimestampFormatError
 from infrahub_sdk.utils import is_valid_url
 from infrahub_sdk.uuidt import UUIDT
 from pydantic import BaseModel, Field
 
 from infrahub import config
 from infrahub.core import registry
+from infrahub.core.changelog.models import AttributeChangelog
 from infrahub.core.constants import NULL_VALUE, AttributeDBNodeType, BranchSupportType, RelationshipStatus
 from infrahub.core.property import FlagPropertyMixin, NodePropertyData, NodePropertyMixin
 from infrahub.core.query.attribute import (
@@ -37,14 +38,12 @@ if TYPE_CHECKING:
     from infrahub.core.schema import AttributeSchema
     from infrahub.database import InfrahubDatabase
 
-# pylint: disable=redefined-builtin,c-extension-no-member,too-many-lines,too-many-public-methods
-
 
 # Use a more user-friendly threshold than Neo4j one (8167 bytes).
 MAX_STRING_LENGTH = 4096
 
 
-def validate_string_length(value: Optional[str]) -> None:
+def validate_string_length(value: str | None) -> None:
     """
     Validates input string length does not exceed a given threshold, as Neo4J cannot index string values larger than 8167 bytes,
     see https://neo4j.com/developer/kb/index-limitations-and-workaround/.
@@ -76,22 +75,22 @@ class AttributeCreateData(BaseModel):
 
 
 class BaseAttribute(FlagPropertyMixin, NodePropertyMixin):
-    type: Optional[Union[type, tuple[type]]] = None
+    type: type | tuple[type] | None = None
 
     _rel_to_node_label: str = RELATIONSHIP_TO_NODE_LABEL
     _rel_to_value_label: str = RELATIONSHIP_TO_VALUE_LABEL
 
-    def __init__(  # pylint: disable=too-many-branches
+    def __init__(
         self,
         name: str,
         schema: AttributeSchema,
         branch: Branch,
         at: Timestamp,
         node: Node,
-        id: Optional[str] = None,
-        db_id: Optional[str] = None,
-        data: Optional[Union[dict, str, AttributeFromDB]] = None,
-        updated_at: Optional[Union[Timestamp, str]] = None,
+        id: str | None = None,
+        db_id: str | None = None,
+        data: dict | str | AttributeFromDB | None = None,
+        updated_at: Timestamp | str | None = None,
         is_default: bool = False,
         is_from_profile: bool = False,
         **kwargs,
@@ -107,7 +106,7 @@ class BaseAttribute(FlagPropertyMixin, NodePropertyMixin):
         self.at = at
         self.is_default = is_default
         self.is_from_profile = is_from_profile
-        self.from_pool: Optional[dict] = None
+        self.from_pool: dict | None = None
 
         self._init_node_property_mixin(kwargs)
         self._init_flag_property_mixin(kwargs)
@@ -218,7 +217,7 @@ class BaseAttribute(FlagPropertyMixin, NodePropertyMixin):
         value_to_check = value
         if schema.enum and isinstance(value, Enum):
             value_to_check = value.value
-        if not isinstance(value_to_check, cls.type):  # pylint: disable=isinstance-second-argument-not-valid-type
+        if not isinstance(value_to_check, cls.type):
             raise ValidationError({name: f"{value} is not a valid {schema.kind}"})
 
     @classmethod
@@ -267,7 +266,7 @@ class BaseAttribute(FlagPropertyMixin, NodePropertyMixin):
     @classmethod
     def deserialize_from_string(cls, value_as_string: str) -> Any:
         """Return a value corresponding to the attribute type given it formatted as a string."""
-        return cls.type(value_as_string)  # pylint: disable=not-callable
+        return cls.type(value_as_string)
 
     def to_db(self) -> dict[str, Any]:
         """Return the properties of the AttributeValue node in Dict format."""
@@ -318,19 +317,19 @@ class BaseAttribute(FlagPropertyMixin, NodePropertyMixin):
         """Deserialize the value coming from the database."""
         return data.value
 
-    async def save(self, db: InfrahubDatabase, at: Optional[Timestamp] = None) -> bool:
+    async def save(self, db: InfrahubDatabase, at: Timestamp | None = None) -> AttributeChangelog | None:
         """Create or Update the Attribute in the database."""
 
         save_at = Timestamp(at)
 
         if not self.id or self.is_from_profile:
-            return False
+            return None
 
         return await self._update(at=save_at, db=db)
 
-    async def delete(self, db: InfrahubDatabase, at: Optional[Timestamp] = None) -> bool:
+    async def delete(self, db: InfrahubDatabase, at: Timestamp | None = None) -> AttributeChangelog | None:
         if not self.db_id:
-            return False
+            return None
 
         delete_at = Timestamp(at)
 
@@ -339,7 +338,14 @@ class BaseAttribute(FlagPropertyMixin, NodePropertyMixin):
         results = query.get_results()
 
         if not results:
-            return False
+            return None
+
+        changelog = AttributeChangelog(
+            name=self.name,
+            value=None,
+            value_previous=None,
+            kind=self.schema.kind,
+        )
 
         properties_to_delete = []
         branch = self.get_branch_based_on_support_type()
@@ -347,6 +353,8 @@ class BaseAttribute(FlagPropertyMixin, NodePropertyMixin):
         # Check all the relationship and update the one that are in the same branch
         rel_ids_to_update = set()
         for result in results:
+            if result.get_rel("r2").type == "HAS_VALUE":
+                changelog.value_previous = result.get_node("ap").get("value")
             properties_to_delete.append((result.get_rel("r2").type, result.get_node("ap").element_id))
 
             await add_relationship(
@@ -378,9 +386,9 @@ class BaseAttribute(FlagPropertyMixin, NodePropertyMixin):
             db=db,
         )
 
-        return True
+        return changelog
 
-    async def _update(self, db: InfrahubDatabase, at: Optional[Timestamp] = None) -> bool:
+    async def _update(self, db: InfrahubDatabase, at: Timestamp | None = None) -> AttributeChangelog | None:
         """Update the attribute in the database.
 
         Get the current value
@@ -420,6 +428,13 @@ class BaseAttribute(FlagPropertyMixin, NodePropertyMixin):
 
         branch = self.get_branch_based_on_support_type()
 
+        changelog = AttributeChangelog(
+            name=self.name,
+            value=self.to_db().get("value"),
+            value_previous=current_attr_data.value,
+            kind=self.schema.kind,
+        )
+
         # ---------- Update the Value ----------
         if current_attr_data.content != self.to_db():
             # Create the new AttributeValue and update the existing relationship
@@ -439,6 +454,11 @@ class BaseAttribute(FlagPropertyMixin, NodePropertyMixin):
 
         for flag_name, _, rel_name in SUPPORTED_FLAGS:
             if current_attr_data.flag_properties[flag_name] != getattr(self, flag_name):
+                changelog.add_property(
+                    name=flag_name,
+                    value_current=getattr(self, flag_name),
+                    value_previous=current_attr_data.flag_properties[flag_name],
+                )
                 query = await AttributeUpdateFlagQuery.init(db=db, attr=self, at=update_at, flag_name=flag_name)
                 await query.execute(db=db)
 
@@ -452,6 +472,16 @@ class BaseAttribute(FlagPropertyMixin, NodePropertyMixin):
                 prop_name in current_attr_data.node_properties
                 and current_attr_data.node_properties[prop_name].uuid == getattr(self, f"{prop_name}_id")
             ):
+                previous_attribute_node_property = current_attr_data.node_properties.get(prop_name)
+                previous_value = None
+                if previous_attribute_node_property:
+                    previous_value = previous_attribute_node_property.uuid
+
+                changelog.add_property(
+                    name=prop_name,
+                    value_current=getattr(self, f"{prop_name}_id"),
+                    value_previous=previous_value,
+                )
                 query = await AttributeUpdateNodePropertyQuery.init(
                     db=db, attr=self, at=update_at, prop_name=prop_name, prop_id=getattr(self, f"{prop_name}_id")
                 )
@@ -461,18 +491,19 @@ class BaseAttribute(FlagPropertyMixin, NodePropertyMixin):
                 if rel and rel.get("branch") == branch.name:
                     await update_relationships_to([rel.element_id], to=update_at, db=db)
 
-        return True
+        if changelog.has_updates:
+            return changelog
 
     async def to_graphql(
         self,
         db: InfrahubDatabase,
-        fields: Optional[dict] = None,
-        related_node_ids: Optional[set] = None,
+        fields: dict | None = None,
+        related_node_ids: set | None = None,
         filter_sensitive: bool = False,
-        permissions: Optional[dict] = None,
+        permissions: dict | None = None,
+        include_properties: bool = True,
     ) -> dict:
         """Generate GraphQL Payload for this attribute."""
-        # pylint: disable=too-many-branches
 
         response: dict[str, Any] = {"id": self.id}
 
@@ -480,7 +511,9 @@ class BaseAttribute(FlagPropertyMixin, NodePropertyMixin):
             field_names = fields.keys()
         else:
             # REMOVED updated_at for now, need to investigate further how it's being used today
-            field_names = ["__typename", "value"] + self._node_properties + self._flag_properties
+            field_names = ["__typename", "value"]
+            if include_properties:
+                field_names += self._node_properties + self._flag_properties
 
         for field_name in field_names:
             if field_name == "updated_at":
@@ -527,11 +560,11 @@ class BaseAttribute(FlagPropertyMixin, NodePropertyMixin):
                     field = field.value
             if isinstance(field, str):
                 response[field_name] = self._filter_sensitive(value=field, filter_sensitive=filter_sensitive)
-            elif isinstance(field, (int, bool, dict, list)):
+            elif isinstance(field, int | bool | dict | list):
                 response[field_name] = field
 
-            if related_node_ids and self.is_from_profile and getattr(self, "source_id"):
-                related_node_ids.add(getattr(self, "source_id"))
+            if related_node_ids and self.is_from_profile and self.source_id:
+                related_node_ids.add(self.source_id)
 
         return response
 
@@ -589,7 +622,6 @@ class BaseAttribute(FlagPropertyMixin, NodePropertyMixin):
         return AttributeDBNodeType.DEFAULT
 
     def get_create_data(self) -> AttributeCreateData:
-        # pylint: disable=no-member
         branch = self.branch
         hierarchy_level = branch.hierarchy_level
         if self.schema.branch == BranchSupportType.AGNOSTIC:
@@ -645,7 +677,7 @@ class String(BaseAttribute):
 
 
 class StringOptional(String):
-    value: Optional[str]
+    value: str | None
 
 
 class HashedPassword(BaseAttribute):
@@ -658,13 +690,13 @@ class HashedPassword(BaseAttribute):
 
 
 class HashedPasswordOptional(HashedPassword):
-    value: Optional[str]
+    value: str | None
 
 
 class Integer(BaseAttribute):
     type = int
     value: int
-    from_pool: Optional[str] = None
+    from_pool: str | None = None
 
     @classmethod
     def validate_format(cls, value: Any, name: str, schema: AttributeSchema) -> None:
@@ -683,7 +715,7 @@ class Integer(BaseAttribute):
 
 
 class IntegerOptional(Integer):
-    value: Optional[int]
+    value: int | None
 
 
 class Boolean(BaseAttribute):
@@ -692,7 +724,7 @@ class Boolean(BaseAttribute):
 
 
 class BooleanOptional(Boolean):
-    value: Optional[bool]
+    value: bool | None
 
 
 class DateTime(BaseAttribute):
@@ -713,7 +745,7 @@ class DateTime(BaseAttribute):
 
 
 class DateTimeOptional(DateTime):
-    value: Optional[str]
+    value: str | None
 
 
 class Dropdown(BaseAttribute):
@@ -760,7 +792,7 @@ class Dropdown(BaseAttribute):
 
 
 class DropdownOptional(Dropdown):
-    value: Optional[str]
+    value: str | None
 
 
 class URL(BaseAttribute):
@@ -776,7 +808,7 @@ class URL(BaseAttribute):
 
 
 class URLOptional(URL):
-    value: Optional[str]
+    value: str | None
 
 
 class IPNetwork(BaseAttribute):
@@ -788,35 +820,35 @@ class IPNetwork(BaseAttribute):
         return ["value", "version", "binary_address", "prefixlen"]
 
     @property
-    def obj(self) -> Union[ipaddress.IPv4Network, ipaddress.IPv6Network]:
+    def obj(self) -> ipaddress.IPv4Network | ipaddress.IPv6Network:
         """Return an ipaddress interface object."""
         if not self.value:
             raise ValueError("value for IPNetwork must be defined")
         return ipaddress.ip_network(str(self.value))
 
     @property
-    def broadcast_address(self) -> Optional[str]:
+    def broadcast_address(self) -> str | None:
         """Return the broadcast address of the ip network."""
         if not self.value:
             return None
         return str(self.obj.broadcast_address)
 
     @property
-    def hostmask(self) -> Optional[str]:
+    def hostmask(self) -> str | None:
         """Return the hostmask of the ip network."""
         if not self.value:
             return None
         return str(self.obj.hostmask)
 
     @property
-    def netmask(self) -> Optional[str]:
+    def netmask(self) -> str | None:
         """Return the netmask of the ip network."""
         if not self.value:
             return None
         return str(self.obj.netmask)
 
     @property
-    def network_address(self) -> Optional[str]:
+    def network_address(self) -> str | None:
         """Return the netmask of the ip network."""
         if not self.value:
             return None
@@ -833,35 +865,35 @@ class IPNetwork(BaseAttribute):
         return convert_ip_to_binary_str(obj=self.obj)
 
     @property
-    def prefixlen(self) -> Optional[int]:
+    def prefixlen(self) -> int | None:
         """Return the prefix length the ip network."""
         if not self.value:
             return None
         return ipaddress.ip_network(str(self.value)).prefixlen
 
     @property
-    def num_addresses(self) -> Optional[int]:
+    def num_addresses(self) -> int | None:
         """Return the number of possible addresses in the ip network."""
         if not self.value:
             return None
         return ipaddress.ip_network(str(self.value)).num_addresses
 
     @property
-    def version(self) -> Optional[int]:
+    def version(self) -> int | None:
         """Return the IP version of the ip network."""
         if not self.value:
             return None
         return ipaddress.ip_network(str(self.value)).version
 
     @property
-    def with_hostmask(self) -> Optional[str]:
+    def with_hostmask(self) -> str | None:
         """Return the network ip and the associated hostmask of the ip network."""
         if not self.value:
             return None
         return ipaddress.ip_network(str(self.value)).with_hostmask
 
     @property
-    def with_netmask(self) -> Optional[str]:
+    def with_netmask(self) -> str | None:
         """Return the network ip and the associated netmask of the ip network."""
         if not self.value:
             return None
@@ -909,7 +941,7 @@ class IPNetwork(BaseAttribute):
 
 
 class IPNetworkOptional(IPNetwork):
-    value: Optional[str]
+    value: str | None
 
 
 class IPHost(BaseAttribute):
@@ -921,63 +953,63 @@ class IPHost(BaseAttribute):
         return ["value", "version", "binary_address"]
 
     @property
-    def obj(self) -> Union[ipaddress.IPv4Interface, ipaddress.IPv6Interface]:
+    def obj(self) -> ipaddress.IPv4Interface | ipaddress.IPv6Interface:
         """Return the ip adress without a prefix or subnet mask."""
         if not self.value:
             raise ValueError("value for IPHost must be defined")
         return ipaddress.ip_interface(str(self.value))
 
     @property
-    def ip(self) -> Optional[str]:
+    def ip(self) -> str | None:
         """Return the ip adress without a prefix or subnet mask."""
         if not self.value:
             return None
         return str(self.obj.ip)
 
     @property
-    def hostmask(self) -> Optional[str]:
+    def hostmask(self) -> str | None:
         """Return the hostmask of the ip address."""
         if not self.value:
             return None
         return str(self.obj.hostmask)
 
     @property
-    def netmask(self) -> Optional[str]:
+    def netmask(self) -> str | None:
         """Return the netmask of the ip address."""
         if not self.value:
             return None
         return str(self.obj.netmask)
 
     @property
-    def network(self) -> Optional[str]:
+    def network(self) -> str | None:
         """Return the network encapsuling the ip address."""
         if not self.value:
             return None
         return str(self.obj.network)
 
     @property
-    def prefixlen(self) -> Optional[int]:
+    def prefixlen(self) -> int | None:
         """Return the prefix length of the ip address."""
         if not self.value:
             return None
         return self.obj.network.prefixlen
 
     @property
-    def version(self) -> Optional[int]:
+    def version(self) -> int | None:
         """Return the IP version of the ip address."""
         if not self.value:
             return None
         return self.obj.version
 
     @property
-    def with_hostmask(self) -> Optional[str]:
+    def with_hostmask(self) -> str | None:
         """Return the ip address and the associated hostmask of the ip address."""
         if not self.value:
             return None
         return self.obj.with_hostmask
 
     @property
-    def with_netmask(self) -> Optional[str]:
+    def with_netmask(self) -> str | None:
         """Return the ip address and the associated netmask of the ip address."""
         if not self.value:
             return None
@@ -1034,7 +1066,7 @@ class IPHost(BaseAttribute):
 
 
 class IPHostOptional(IPHost):
-    value: Optional[str]
+    value: str | None
 
 
 class MacAddress(BaseAttribute):
@@ -1049,7 +1081,7 @@ class MacAddress(BaseAttribute):
         return netaddr.EUI(addr=self.value)
 
     @property
-    def oui(self) -> Optional[str]:
+    def oui(self) -> str | None:
         """Return the OUI (Organisationally Unique Identifier) for the MAC address."""
         if not self.value:
             return None
@@ -1061,58 +1093,58 @@ class MacAddress(BaseAttribute):
             return str(self.obj).removesuffix(f"-{self.ei}")
 
     @property
-    def ei(self) -> Optional[str]:
+    def ei(self) -> str | None:
         """Return the EI (Extension Identifier) for the MAC address."""
         if not self.value:
             return None
         return self.obj.ei
 
     @property
-    def version(self) -> Optional[int]:
+    def version(self) -> int | None:
         """Return the version of the MAC address."""
         if not self.value:
             return None
         return self.obj.version
 
     @property
-    def binary(self) -> Optional[str]:
+    def binary(self) -> str | None:
         """Return the MAC address in binary format."""
         if not self.value:
             return None
         return self.obj.bin
 
     @property
-    def eui48(self) -> Optional[str]:
+    def eui48(self) -> str | None:
         if not self.value:
             return None
         return self.obj.format(dialect=netaddr.mac_eui48)
 
     @property
-    def eui64(self) -> Optional[str]:
+    def eui64(self) -> str | None:
         if not self.value:
             return None
         return str(self.obj.eui64())
 
     @property
-    def bare(self) -> Optional[str]:
+    def bare(self) -> str | None:
         if not self.value:
             return None
         return self.obj.format(dialect=netaddr.mac_bare)
 
     @property
-    def dot_notation(self) -> Optional[str]:
+    def dot_notation(self) -> str | None:
         if not self.value:
             return None
         return self.obj.format(dialect=netaddr.mac_cisco)
 
     @property
-    def semicolon_notation(self) -> Optional[str]:
+    def semicolon_notation(self) -> str | None:
         if not self.value:
             return None
         return self.obj.format(dialect=netaddr.mac_unix)
 
     @property
-    def split_notation(self) -> Optional[str]:
+    def split_notation(self) -> str | None:
         if not self.value:
             return None
         return self.obj.format(dialect=netaddr.mac_pgsql)
@@ -1140,7 +1172,7 @@ class MacAddress(BaseAttribute):
 
 
 class MacAddressOptional(MacAddress):
-    value: Optional[str]
+    value: str | None
 
 
 class ListAttribute(BaseAttribute):
@@ -1159,18 +1191,29 @@ class ListAttribute(BaseAttribute):
 
     def deserialize_value(self, data: AttributeFromDB) -> Any:
         """Deserialize the value (potentially) coming from the database."""
-        if isinstance(data.value, (str, bytes)):
+        if isinstance(data.value, str | bytes):
             return ujson.loads(data.value)
         return data.value
 
+    @classmethod
+    def validate_format(cls, value: Any, name: str, schema: AttributeSchema) -> None:
+        value_to_check = value
+
+        if isinstance(value, str):
+            try:
+                value_to_check = cls.deserialize_from_string(value_as_string=value)
+            except ujson.JSONDecodeError as exc:
+                raise ValidationError({name: f"{value} is not a valid JSON list"}) from exc
+        super().validate_format(value=value_to_check, name=name, schema=schema)
+
 
 class ListAttributeOptional(ListAttribute):
-    value: Optional[list[Any]]
+    value: list[Any] | None
 
 
 class JSONAttribute(BaseAttribute):
     type = (dict, list)
-    value: Union[dict, list]
+    value: dict | list
 
     @classmethod
     def deserialize_from_string(cls, value_as_string: str) -> Any:
@@ -1184,10 +1227,21 @@ class JSONAttribute(BaseAttribute):
 
     def deserialize_value(self, data: AttributeFromDB) -> Any:
         """Deserialize the value (potentially) coming from the database."""
-        if data.value and isinstance(data.value, (str, bytes)):
+        if data.value and isinstance(data.value, str | bytes):
             return ujson.loads(data.value)
         return data.value
 
+    @classmethod
+    def validate_format(cls, value: Any, name: str, schema: AttributeSchema) -> None:
+        value_to_check = value
+
+        if isinstance(value, str):
+            try:
+                value_to_check = cls.deserialize_from_string(value_as_string=value)
+            except ujson.JSONDecodeError as exc:
+                raise ValidationError({name: f"{value} is not valid JSON"}) from exc
+        super().validate_format(value=value_to_check, name=name, schema=schema)
+
 
 class JSONAttributeOptional(JSONAttribute):
-    value: Optional[Union[dict, list]]
+    value: dict | list | None
