@@ -1,9 +1,8 @@
 from __future__ import annotations
 
-from dataclasses import asdict, dataclass, field, replace
+from dataclasses import asdict, dataclass, field
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Optional
-from uuid import uuid4
+from typing import TYPE_CHECKING, Any
 
 from infrahub.core.constants import (
     BranchSupportType,
@@ -21,7 +20,7 @@ if TYPE_CHECKING:
     from neo4j.graph import Node as Neo4jNode
     from neo4j.graph import Path as Neo4jPath
     from neo4j.graph import Relationship as Neo4jRelationship
-    from pendulum import Interval
+    from whenever import TimeDelta
 
     from infrahub.graphql.initialization import GraphqlContext
 
@@ -82,15 +81,6 @@ def deserialize_tracking_id(tracking_id_str: str) -> TrackingId:
     raise ValueError(f"{tracking_id_str} is not a valid TrackingId")
 
 
-@dataclass
-class NodeFieldSpecifier:
-    node_uuid: str
-    field_name: str
-
-    def __hash__(self) -> int:
-        return hash(f"{self.node_uuid}:{self.field_name}")
-
-
 @dataclass
 class NodeDiffFieldSummary:
     kind: str
@@ -173,6 +163,10 @@ class EnrichedDiffAttribute(BaseSummary):
     def __hash__(self) -> int:
         return hash(self.name)
 
+    @property
+    def num_properties(self) -> int:
+        return len(self.properties)
+
     def get_all_conflicts(self) -> dict[str, EnrichedDiffConflict]:
         return {prop.path_identifier: prop.conflict for prop in self.properties if prop.conflict}
 
@@ -180,6 +174,12 @@ class EnrichedDiffAttribute(BaseSummary):
         for prop in self.properties:
             prop.conflict = None
 
+    def get_property(self, property_type: DatabaseEdgeType) -> EnrichedDiffProperty:
+        for prop in self.properties:
+            if prop.property_type is property_type:
+                return prop
+        raise ValueError(f"No {property_type.value} property found")
+
     @classmethod
     def from_calculated_attribute(cls, calculated_attribute: DiffAttribute) -> EnrichedDiffAttribute:
         return EnrichedDiffAttribute(
@@ -206,6 +206,10 @@ class EnrichedDiffSingleRelationship(BaseSummary):
     def __hash__(self) -> int:
         return hash(self.peer_id)
 
+    @property
+    def num_properties(self) -> int:
+        return len(self.properties)
+
     def get_all_conflicts(self) -> dict[str, EnrichedDiffConflict]:
         all_conflicts: dict[str, EnrichedDiffConflict] = {}
         if self.conflict:
@@ -252,6 +256,10 @@ class EnrichedDiffRelationship(BaseSummary):
     def __hash__(self) -> int:
         return hash(self.name)
 
+    @property
+    def num_properties(self) -> int:
+        return sum(r.num_properties for r in self.relationships)
+
     def get_all_conflicts(self) -> dict[str, EnrichedDiffConflict]:
         all_conflicts: dict[str, EnrichedDiffConflict] = {}
         for element in self.relationships:
@@ -262,6 +270,12 @@ class EnrichedDiffRelationship(BaseSummary):
         for element in self.relationships:
             element.clear_conflicts()
 
+    def get_element(self, peer_id: str) -> EnrichedDiffSingleRelationship:
+        for element in self.relationships:
+            if element.peer_id == peer_id:
+                return element
+        raise ValueError(f"No relationship for {peer_id} found")
+
     @property
     def include_in_response(self) -> bool:
         if self.action == DiffAction.UNCHANGED and not self.relationships:
@@ -306,6 +320,10 @@ class EnrichedDiffNode(BaseSummary):
     def __hash__(self) -> int:
         return hash(self.uuid)
 
+    @property
+    def num_properties(self) -> int:
+        return sum(a.num_properties for a in self.attributes) + sum(r.num_properties for r in self.relationships)
+
     def get_all_conflicts(self) -> dict[str, EnrichedDiffConflict]:
         all_conflicts: dict[str, EnrichedDiffConflict] = {}
         if self.conflict:
@@ -323,18 +341,18 @@ class EnrichedDiffNode(BaseSummary):
             rel.clear_conflicts()
         self.conflict = None
 
-    def get_parent_info(self, context: GraphqlContext | None = None) -> ParentNodeInfo | None:
+    def get_parent_info(self, graphql_context: GraphqlContext | None = None) -> ParentNodeInfo | None:
         for r in self.relationships:
             for n in r.nodes:
                 relationship_name: str = "undefined"
 
-                if not context:
+                if not graphql_context:
                     return ParentNodeInfo(node=n, relationship_name=relationship_name)
 
-                node_schema = context.db.schema.get(name=self.kind)
+                node_schema = graphql_context.db.schema.get(name=self.kind)
                 rel_schema = node_schema.get_relationship(name=r.name)
 
-                parent_schema = context.db.schema.get(name=n.kind)
+                parent_schema = graphql_context.db.schema.get(name=n.kind)
                 rels_parent = parent_schema.get_relationships_by_identifier(id=rel_schema.get_identifier())
 
                 if rels_parent and len(rels_parent) == 1:
@@ -362,16 +380,11 @@ class EnrichedDiffNode(BaseSummary):
                 all_children |= n.get_all_child_nodes()
         return all_children
 
-    def get_trimmed_node(self, max_depth: int) -> EnrichedDiffNode:
-        trimmed = replace(self, relationships=set())
-        for rel in self.relationships:
-            trimmed_rel = replace(rel, nodes=set())
-            trimmed.relationships.add(trimmed_rel)
-            if max_depth == 0:
-                continue
-            for child_node in rel.nodes:
-                trimmed_rel.nodes.add(child_node.get_trimmed_node(max_depth=max_depth - 1))
-        return trimmed
+    def get_attribute(self, name: str) -> EnrichedDiffAttribute:
+        for attr in self.attributes:
+            if attr.name == name:
+                return attr
+        raise ValueError(f"No attribute {name} found")
 
     def get_relationship(self, name: str) -> EnrichedDiffRelationship:
         for rel in self.relationships:
@@ -412,15 +425,16 @@ class EnrichedDiffRootMetadata(BaseSummary):
     from_time: Timestamp
     to_time: Timestamp
     uuid: str
-    partner_uuid: str
-    tracking_id: TrackingId | None = field(default=None, kw_only=True)
+    tracking_id: TrackingId
+    partner_uuid: str | None = field(default=None)
+    exists_on_database: bool = field(default=False)
 
     def __hash__(self) -> int:
         return hash(self.uuid)
 
     @property
-    def time_range(self) -> Interval:
-        return self.to_time.obj - self.from_time.obj
+    def time_range(self) -> TimeDelta:
+        return self.to_time.get_obj() - self.from_time.get_obj()
 
     def update_metadata(
         self,
@@ -435,7 +449,7 @@ class EnrichedDiffRootMetadata(BaseSummary):
         if to_time and self.to_time != to_time:
             self.to_time = to_time
             is_changed = True
-        if self.tracking_id != tracking_id:
+        if self.tracking_id != tracking_id and tracking_id is not None:
             self.tracking_id = tracking_id
             is_changed = True
         return is_changed
@@ -449,8 +463,8 @@ class EnrichedDiffRoot(EnrichedDiffRootMetadata):
         return hash(self.uuid)
 
     @property
-    def time_range(self) -> Interval:
-        return self.to_time.obj - self.from_time.obj
+    def time_range(self) -> TimeDelta:
+        return self.to_time.get_obj() - self.from_time.get_obj()
 
     def get_nodes_without_parents(self) -> set[EnrichedDiffNode]:
         nodes_with_parent_uuids = set()
@@ -472,6 +486,13 @@ class EnrichedDiffRoot(EnrichedDiffRootMetadata):
         except ValueError:
             return False
 
+    def get_node_map(self, node_uuids: set[str] | None = None) -> dict[str, EnrichedDiffNode]:
+        node_map = {}
+        for node in self.nodes:
+            if node_uuids is None or node.uuid in node_uuids:
+                node_map[node.uuid] = node
+        return node_map
+
     def get_all_conflicts(self) -> dict[str, EnrichedDiffConflict]:
         all_conflicts: dict[str, EnrichedDiffConflict] = {}
         for node in self.nodes:
@@ -484,7 +505,11 @@ class EnrichedDiffRoot(EnrichedDiffRootMetadata):
 
     @classmethod
     def from_calculated_diff(
-        cls, calculated_diff: DiffRoot, base_branch_name: str, partner_uuid: str
+        cls,
+        calculated_diff: DiffRoot,
+        base_branch_name: str,
+        partner_uuid: str,
+        tracking_id: TrackingId,
     ) -> EnrichedDiffRoot:
         return EnrichedDiffRoot(
             base_branch_name=base_branch_name,
@@ -493,6 +518,7 @@ class EnrichedDiffRoot(EnrichedDiffRootMetadata):
             to_time=calculated_diff.to_time,
             uuid=calculated_diff.uuid,
             partner_uuid=partner_uuid,
+            tracking_id=tracking_id,
             nodes={EnrichedDiffNode.from_calculated_node(calculated_node=n) for n in calculated_diff.nodes},
         )
 
@@ -572,14 +598,6 @@ class EnrichedDiffsMetadata:
         )
         return is_changed
 
-    def set_fresh_uuids(self) -> None:
-        base_uuid = str(uuid4())
-        branch_uuid = str(uuid4())
-        self.base_branch_diff.uuid = base_uuid
-        self.base_branch_diff.partner_uuid = branch_uuid
-        self.diff_branch_diff.uuid = branch_uuid
-        self.diff_branch_diff.partner_uuid = base_uuid
-
 
 @dataclass
 class EnrichedDiffs(EnrichedDiffsMetadata):
@@ -594,20 +612,24 @@ class EnrichedDiffs(EnrichedDiffsMetadata):
             f"branch_name={self.diff_branch_name},"
             f"base_name={self.base_branch_name},"
             f"from_time={self.diff_branch_diff.from_time},"
-            f"to_time={self.diff_branch_diff.to_time})"
+            f"to_time={self.diff_branch_diff.to_time},"
+            f"num_base_nodes={len(self.base_branch_diff.nodes)},"
+            f"num_branch_nodes={len(self.diff_branch_diff.nodes)}"
         )
 
     @classmethod
-    def from_calculated_diffs(cls, calculated_diffs: CalculatedDiffs) -> EnrichedDiffs:
+    def from_calculated_diffs(cls, calculated_diffs: CalculatedDiffs, tracking_id: TrackingId) -> EnrichedDiffs:
         base_branch_diff = EnrichedDiffRoot.from_calculated_diff(
             calculated_diff=calculated_diffs.base_branch_diff,
             base_branch_name=calculated_diffs.base_branch_name,
             partner_uuid=calculated_diffs.diff_branch_diff.uuid,
+            tracking_id=tracking_id,
         )
         diff_branch_diff = EnrichedDiffRoot.from_calculated_diff(
             calculated_diff=calculated_diffs.diff_branch_diff,
             base_branch_name=calculated_diffs.base_branch_name,
             partner_uuid=calculated_diffs.base_branch_diff.uuid,
+            tracking_id=tracking_id,
         )
         return EnrichedDiffs(
             base_branch_name=calculated_diffs.base_branch_name,
@@ -620,6 +642,14 @@ class EnrichedDiffs(EnrichedDiffsMetadata):
     def is_empty(self) -> bool:
         return len(self.base_branch_diff.nodes) == 0 and len(self.diff_branch_diff.nodes) == 0
 
+    @property
+    def base_node_uuids(self) -> set[str]:
+        return {n.uuid for n in self.base_branch_diff.nodes}
+
+    @property
+    def branch_node_uuids(self) -> set[str]:
+        return {n.uuid for n in self.diff_branch_diff.nodes}
+
 
 @dataclass
 class CalculatedDiffs:
@@ -685,7 +715,7 @@ class DiffRoot:
 
 
 @dataclass
-class DatabasePath:  # pylint: disable=too-many-public-methods
+class DatabasePath:
     root_node: Neo4jNode
     path_to_node: Neo4jRelationship
     node_node: Neo4jNode
@@ -746,6 +776,10 @@ class DatabasePath:  # pylint: disable=too-many-public-methods
     def node_id(self) -> str:
         return str(self.node_node.get("uuid"))
 
+    @property
+    def node_db_id(self) -> str:
+        return self.node_node.element_id
+
     @property
     def node_kind(self) -> str:
         return str(self.node_node.get("kind"))
@@ -814,13 +848,13 @@ class DatabasePath:  # pylint: disable=too-many-public-methods
         return "Node" in self.property_node.labels
 
     @property
-    def peer_id(self) -> Optional[str]:
+    def peer_id(self) -> str | None:
         if not self.property_is_peer:
             return None
         return str(self.property_node.get("uuid"))
 
     @property
-    def peer_kind(self) -> Optional[str]:
+    def peer_kind(self) -> str | None:
         if not self.property_is_peer:
             return None
         return str(self.property_node.get("kind"))
