@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 from functools import reduce
-from typing import TYPE_CHECKING, Any, Iterable, Literal, Optional, TypeVar, Union, overload
+from typing import TYPE_CHECKING, Any, Iterable, Literal, TypeVar, overload
 
 from infrahub_sdk.utils import deep_merge_dict, is_valid_uuid
 
@@ -23,7 +23,14 @@ from infrahub.core.query.node import (
 from infrahub.core.query.relationship import RelationshipGetPeerQuery
 from infrahub.core.registry import registry
 from infrahub.core.relationship import Relationship, RelationshipManager
-from infrahub.core.schema import GenericSchema, MainSchemaTypes, NodeSchema, ProfileSchema, RelationshipSchema
+from infrahub.core.schema import (
+    GenericSchema,
+    MainSchemaTypes,
+    NodeSchema,
+    ProfileSchema,
+    RelationshipSchema,
+    TemplateSchema,
+)
 from infrahub.core.timestamp import Timestamp
 from infrahub.exceptions import NodeNotFoundError, ProcessingError, SchemaNotFoundError
 from infrahub.graphql.models import OrderModel
@@ -35,8 +42,6 @@ if TYPE_CHECKING:
 
 SchemaProtocol = TypeVar("SchemaProtocol")
 
-# pylint: disable=redefined-builtin,too-many-lines
-
 
 def identify_node_class(node: NodeToProcess) -> type[Node]:
     """Identify the proper class to use to create the NodeObject.
@@ -61,7 +66,7 @@ def get_schema(
 ) -> MainSchemaTypes:
     if isinstance(node_schema, str):
         return db.schema.get(name=node_schema, branch=branch.name)
-    if hasattr(node_schema, "_is_runtime_protocol") and getattr(node_schema, "_is_runtime_protocol"):
+    if hasattr(node_schema, "_is_runtime_protocol") and node_schema._is_runtime_protocol:
         return db.schema.get(name=node_schema.__name__, branch=branch.name)
     if not isinstance(node_schema, (MainSchemaTypes)):
         raise ValueError(f"Invalid schema provided {node_schema}")
@@ -88,7 +93,7 @@ class ProfileAttributeIndex:
             self._profile_attributes_id_map[p_id] for p_id in profile_ids if p_id in self._profile_attributes_id_map
         ]
 
-        def get_profile_priority(nafd: NodeAttributesFromDB) -> tuple[Union[int, float], str]:
+        def get_profile_priority(nafd: NodeAttributesFromDB) -> tuple[int | float, str]:
             try:
                 return (int(nafd.attrs.get("profile_priority").value), nafd.node.get("uuid"))
             except (TypeError, AttributeError):
@@ -129,13 +134,13 @@ class NodeManager:
     async def query(
         cls,
         db: InfrahubDatabase,
-        schema: Union[NodeSchema, GenericSchema, ProfileSchema, str],
+        schema: NodeSchema | GenericSchema | ProfileSchema | TemplateSchema | str,
         filters: dict | None = ...,
         fields: dict | None = ...,
         offset: int | None = ...,
         limit: int | None = ...,
-        at: Union[Timestamp, str] | None = ...,
-        branch: Union[Branch, str] | None = ...,
+        at: Timestamp | str | None = ...,
+        branch: Branch | str | None = ...,
         include_source: bool = ...,
         include_owner: bool = ...,
         prefetch_relationships: bool = ...,
@@ -155,8 +160,8 @@ class NodeManager:
         fields: dict | None = ...,
         offset: int | None = ...,
         limit: int | None = ...,
-        at: Union[Timestamp, str] | None = ...,
-        branch: Union[Branch, str] | None = ...,
+        at: Timestamp | str | None = ...,
+        branch: Branch | str | None = ...,
         include_source: bool = ...,
         include_owner: bool = ...,
         prefetch_relationships: bool = ...,
@@ -175,8 +180,8 @@ class NodeManager:
         fields: dict | None = None,
         offset: int | None = None,
         limit: int | None = None,
-        at: Union[Timestamp, str] | None = None,
-        branch: Union[Branch, str] | None = None,
+        at: Timestamp | str | None = None,
+        branch: Branch | str | None = None,
         include_source: bool = False,
         include_owner: bool = False,
         prefetch_relationships: bool = False,
@@ -267,11 +272,11 @@ class NodeManager:
     async def count(
         cls,
         db: InfrahubDatabase,
-        schema: Union[type[SchemaProtocol], NodeSchema, GenericSchema, ProfileSchema, str],
-        filters: Optional[dict] = None,
-        at: Optional[Union[Timestamp, str]] = None,
-        branch: Optional[Union[Branch, str]] = None,
-        account=None,  # pylint: disable=unused-argument
+        schema: type[SchemaProtocol] | NodeSchema | GenericSchema | ProfileSchema | TemplateSchema | str,
+        filters: dict | None = None,
+        at: Timestamp | str | None = None,
+        branch: Branch | str | None = None,
+        account=None,  # noqa: ARG003
         partial_match: bool = False,
         branch_agnostic: bool = False,
     ) -> int:
@@ -312,8 +317,8 @@ class NodeManager:
         schema: RelationshipSchema,
         filters: dict,
         db: InfrahubDatabase,
-        at: Optional[Union[Timestamp, str]] = None,
-        branch: Optional[Union[Branch, str]] = None,
+        at: Timestamp | str | None = None,
+        branch: Branch | str | None = None,
         branch_agnostic: bool = False,
     ) -> int:
         branch = await registry.get_branch(branch=branch, db=db)
@@ -341,11 +346,11 @@ class NodeManager:
         source_kind: str,
         schema: RelationshipSchema,
         filters: dict,
-        fields: Optional[dict] = None,
-        offset: Optional[int] = None,
-        limit: Optional[int] = None,
-        at: Optional[Union[Timestamp, str]] = None,
-        branch: Optional[Union[Branch, str]] = None,
+        fields: dict | None = None,
+        offset: int | None = None,
+        limit: int | None = None,
+        at: Timestamp | str | None = None,
+        branch: Branch | str | None = None,
         branch_agnostic: bool = False,
         fetch_peers: bool = False,
     ) -> list[Relationship]:
@@ -415,8 +420,8 @@ class NodeManager:
         node_schema: NodeSchema,
         filters: dict,
         db: InfrahubDatabase,
-        at: Optional[Union[Timestamp, str]] = None,
-        branch: Optional[Union[Branch, str]] = None,
+        at: Timestamp | str | None = None,
+        branch: Branch | str | None = None,
     ) -> int:
         branch = await registry.get_branch(branch=branch, db=db)
         at = Timestamp(at)
@@ -441,12 +446,12 @@ class NodeManager:
         direction: RelationshipHierarchyDirection,
         node_schema: NodeSchema,
         filters: dict,
-        fields: Optional[dict] = None,
-        offset: Optional[int] = None,
-        limit: Optional[int] = None,
-        at: Optional[Union[Timestamp, str]] = None,
-        branch: Optional[Union[Branch, str]] = None,
-    ) -> dict[str, Any]:
+        fields: dict | None = None,
+        offset: int | None = None,
+        limit: int | None = None,
+        at: Timestamp | str | None = None,
+        branch: Branch | str | None = None,
+    ) -> dict[str, Node]:
         branch = await registry.get_branch(branch=branch, db=db)
         at = Timestamp(at)
 
@@ -489,7 +494,7 @@ class NodeManager:
         kind: type[SchemaProtocol],
         at: Timestamp | str | None = ...,
         branch: Branch | str | None = ...,
-        id: str | None = ...,  # pylint: disable=redefined-builtin
+        id: str | None = ...,
         hfid: list[str] | None = ...,
     ) -> SchemaProtocol: ...
 
@@ -501,7 +506,7 @@ class NodeManager:
         kind: str,
         at: Timestamp | str | None = ...,
         branch: Branch | str | None = ...,
-        id: str | None = ...,  # pylint: disable=redefined-builtin
+        id: str | None = ...,
         hfid: list[str] | None = ...,
     ) -> Any: ...
 
@@ -512,7 +517,7 @@ class NodeManager:
         kind: type[SchemaProtocol] | str,
         at: Timestamp | str | None = None,
         branch: Branch | str | None = None,
-        id: str | None = None,  # pylint: disable=redefined-builtin
+        id: str | None = None,
         hfid: list[str] | None = None,
     ) -> Any:
         if not id and not hfid:
@@ -802,7 +807,7 @@ class NodeManager:
             raise NodeNotFoundError(branch_name=branch.name, node_type=kind_str, identifier=hfid_str)
 
         filters = {}
-        for key, item in zip(node_schema.human_friendly_id, hfid):
+        for key, item in zip(node_schema.human_friendly_id, hfid, strict=False):
             path = node_schema.parse_schema_path(path=key, schema=registry.schema.get_schema_branch(name=branch.name))
 
             if path.is_type_relationship:
@@ -1007,7 +1012,7 @@ class NodeManager:
         cls,
         id: str,
         db: InfrahubDatabase,
-        kind: Literal[None] = ...,
+        kind: None = ...,
         raise_on_error: bool = ...,
         fields: dict | None = ...,
         at: Timestamp | str | None = ...,
@@ -1034,7 +1039,7 @@ class NodeManager:
         prefetch_relationships: bool = False,
         account=None,
         branch_agnostic: bool = False,
-    ) -> Optional[Any]:
+    ) -> Any | None:
         """Return one node based on its ID."""
         branch = await registry.get_branch(branch=branch, db=db)
 
@@ -1090,9 +1095,9 @@ class NodeManager:
         cls,
         db: InfrahubDatabase,
         ids: list[str],
-        fields: Optional[dict] = None,
-        at: Optional[Union[Timestamp, str]] = None,
-        branch: Optional[Union[Branch, str]] = None,
+        fields: dict | None = None,
+        at: Timestamp | str | None = None,
+        branch: Branch | str | None = None,
         include_source: bool = False,
         include_owner: bool = False,
         prefetch_relationships: bool = False,
@@ -1148,12 +1153,12 @@ class NodeManager:
 
         nodes: dict[str, Node] = {}
 
-        for node_id in ids:  # pylint: disable=too-many-nested-blocks
+        for node_id in ids:
             if node_id not in nodes_info_by_id:
                 continue
 
             node = nodes_info_by_id[node_id]
-            new_node_data: dict[str, Union[str, AttributeFromDB]] = {
+            new_node_data: dict[str, str | AttributeFromDB] = {
                 "db_id": node.node_id,
                 "id": node_id,
                 "updated_at": node.updated_at,
@@ -1308,9 +1313,9 @@ class NodeManager:
         cls,
         db: InfrahubDatabase,
         nodes: list[Node],
-        branch: Optional[Union[Branch, str]] = None,
-        at: Optional[Union[Timestamp, str]] = None,
-    ) -> list[Any]:
+        branch: Branch | str | None = None,
+        at: Timestamp | str | None = None,
+    ) -> list[Node]:
         """Returns list of deleted nodes because of cascading deletes"""
         branch = await registry.get_branch(branch=branch, db=db)
         node_delete_validator = NodeDeleteValidator(db=db, branch=branch)
