@@ -1,5 +1,3 @@
-from typing import Optional
-
 from .schema.attribute_name_update import AttributeNameUpdateMigration
 from .schema.node_attribute_add import NodeAttributeAddMigration
 from .schema.node_attribute_remove import NodeAttributeRemoveMigration
@@ -8,7 +6,7 @@ from .schema.node_remove import NodeRemoveMigration
 from .schema.placeholder_dummy import PlaceholderDummyMigration
 from .shared import SchemaMigration
 
-MIGRATION_MAP: dict[str, Optional[type[SchemaMigration]]] = {
+MIGRATION_MAP: dict[str, type[SchemaMigration] | None] = {
     "node.remove": NodeRemoveMigration,
     "node.branch.update": None,
     "node.attribute.add": NodeAttributeAddMigration,
