@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Sequence, Union
+from typing import TYPE_CHECKING, Sequence
 
 from .m001_add_version_to_graph import Migration001
 from .m002_attribute_is_default import Migration002
@@ -20,13 +20,17 @@ from .m015_diff_format_update import Migration015
 from .m016_diff_delete_bug_fix import Migration016
 from .m017_add_core_profile import Migration017
 from .m018_uniqueness_nulls import Migration018
+from .m019_restore_rels_to_time import Migration019
+from .m020_duplicate_edges import Migration020
+from .m021_missing_hierarchy_merge import Migration021
+from .m022_add_generate_template_attr import Migration022
 
 if TYPE_CHECKING:
     from infrahub.core.root import Root
 
     from ..shared import ArbitraryMigration, GraphMigration, InternalSchemaMigration
 
-MIGRATIONS: list[type[Union[GraphMigration, InternalSchemaMigration, ArbitraryMigration]]] = [
+MIGRATIONS: list[type[GraphMigration | InternalSchemaMigration | ArbitraryMigration]] = [
     Migration001,
     Migration002,
     Migration003,
@@ -45,12 +49,16 @@ MIGRATIONS: list[type[Union[GraphMigration, InternalSchemaMigration, ArbitraryMi
     Migration016,
     Migration017,
     Migration018,
+    Migration019,
+    Migration020,
+    Migration021,
+    Migration022,
 ]
 
 
 async def get_graph_migrations(
     root: Root,
-) -> Sequence[Union[GraphMigration, InternalSchemaMigration, ArbitraryMigration]]:
+) -> Sequence[GraphMigration | InternalSchemaMigration | ArbitraryMigration]:
     applicable_migrations = []
     for migration_class in MIGRATIONS:
         migration = migration_class.init()
