@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import asyncio
 import copy
 import sys
 from dataclasses import dataclass
@@ -10,10 +11,8 @@ from typing import (
     Iterator,
     Literal,
     Mapping,
-    Optional,
     Sequence,
     TypeVar,
-    Union,
     overload,
 )
 
@@ -22,7 +21,8 @@ from infrahub_sdk.uuidt import UUIDT
 from pydantic import BaseModel, Field
 
 from infrahub.core import registry
-from infrahub.core.constants import BranchSupportType, InfrahubKind
+from infrahub.core.changelog.models import ChangelogRelationshipMapper
+from infrahub.core.constants import BranchSupportType, InfrahubKind, RelationshipKind
 from infrahub.core.property import (
     FlagPropertyMixin,
     NodePropertyData,
@@ -47,11 +47,11 @@ if TYPE_CHECKING:
     from typing_extensions import Self
 
     from infrahub.core.branch import Branch
+    from infrahub.core.changelog.models import RelationshipCardinalityManyChangelog, RelationshipCardinalityOneChangelog
     from infrahub.core.node import Node
     from infrahub.core.schema import MainSchemaTypes, RelationshipSchema
     from infrahub.database import InfrahubDatabase
 
-# pylint: disable=redefined-builtin,too-many-lines
 
 PeerType = TypeVar("PeerType")
 
@@ -63,14 +63,14 @@ class RelationshipCreateData(BaseModel):
     uuid: str
     name: str
     destination_id: str
-    branch: Optional[str] = None
+    branch: str | None = None
     branch_level: int
-    branch_support: Optional[str] = None
+    branch_support: str | None = None
     direction: str
     status: str
     is_protected: bool
     is_visible: bool
-    hierarchical: Optional[str] = None
+    hierarchical: str | None = None
     source_prop: list[NodePropertyData] = Field(default_factory=list)
     owner_prop: list[NodePropertyData] = Field(default_factory=list)
 
@@ -90,9 +90,9 @@ class Relationship(FlagPropertyMixin, NodePropertyMixin):
         self,
         schema: RelationshipSchema,
         branch: Branch,
-        at: Optional[Timestamp] = None,
-        node: Optional[Node] = None,
-        node_id: Optional[str] = None,
+        at: Timestamp | None = None,
+        node: Node | None = None,
+        node_id: str | None = None,
         **kwargs: Any,
     ) -> None:
         if not node and not node_id:
@@ -105,18 +105,18 @@ class Relationship(FlagPropertyMixin, NodePropertyMixin):
         self.at = Timestamp(at)
 
         self._node = node
-        self._node_id: Optional[str] = node_id
+        self._node_id: str | None = node_id
 
-        self.id: Optional[UUID] = None
-        self.db_id: Optional[str] = None
-        self.updated_at: Optional[Timestamp] = None
+        self.id: UUID | None = None
+        self.db_id: str | None = None
+        self.updated_at: Timestamp | None = None
 
-        self._peer: Optional[Union[Node, str]] = None
-        self.peer_id: Optional[str] = None
-        self.peer_hfid: Optional[list[str]] = None
-        self.data: Optional[Union[dict, RelationshipPeerData, str]] = None
+        self._peer: Node | str | None = None
+        self.peer_id: str | None = None
+        self.peer_hfid: list[str] | None = None
+        self.data: dict | RelationshipPeerData | str | None = None
 
-        self.from_pool: Optional[dict[str, Any]] = None
+        self.from_pool: dict[str, Any] | None = None
 
         self._init_node_property_mixin(kwargs=kwargs)
         self._init_flag_property_mixin(kwargs=kwargs)
@@ -139,6 +139,12 @@ class Relationship(FlagPropertyMixin, NodePropertyMixin):
 
         return self.peer_id
 
+    def get_peer_kind(self) -> str:
+        if not self._peer or isinstance(self._peer, str):
+            return self.schema.peer
+
+        return self._peer.get_kind()
+
     @property
     def node_id(self) -> str:
         if self._node_id:
@@ -158,7 +164,7 @@ class Relationship(FlagPropertyMixin, NodePropertyMixin):
             return registry.get_global_branch()
         return self.branch
 
-    async def _process_data(self, data: Union[dict, RelationshipPeerData, str]) -> None:  # pylint: disable=too-many-branches
+    async def _process_data(self, data: dict | RelationshipPeerData | str) -> None:
         self.data = data
 
         if isinstance(data, RelationshipPeerData):
@@ -192,20 +198,23 @@ class Relationship(FlagPropertyMixin, NodePropertyMixin):
         else:
             await self.set_peer(value=data)
 
-    async def new(  # pylint: disable=unused-argument
-        self, db: InfrahubDatabase, data: Union[dict, RelationshipPeerData, Any] = None, **kwargs: Any
+    async def new(
+        self,
+        db: InfrahubDatabase,  # noqa: ARG002
+        data: dict | RelationshipPeerData | Any = None,
+        **kwargs: Any,  # noqa: ARG002
     ) -> Relationship:
         await self._process_data(data=data)
 
         return self
 
-    async def load(  # pylint: disable=unused-argument
+    async def load(
         self,
-        db: InfrahubDatabase,
-        id: Optional[UUID] = None,
-        db_id: Optional[str] = None,
-        updated_at: Optional[Union[Timestamp, str]] = None,
-        data: Union[dict, RelationshipPeerData, Any] = None,
+        db: InfrahubDatabase,  # noqa: ARG002
+        id: UUID | None = None,
+        db_id: str | None = None,
+        updated_at: Timestamp | str | None = None,
+        data: dict | RelationshipPeerData | Any = None,
     ) -> Self:
         hash_before = hash(self)
 
@@ -241,7 +250,7 @@ class Relationship(FlagPropertyMixin, NodePropertyMixin):
         self._node_id = self._node.id
         return node
 
-    async def set_peer(self, value: Union[str, Node]) -> None:
+    async def set_peer(self, value: str | Node) -> None:
         if isinstance(value, str):
             self.peer_id = value
         else:
@@ -252,9 +261,9 @@ class Relationship(FlagPropertyMixin, NodePropertyMixin):
     async def get_peer(self, db: InfrahubDatabase, peer_type: type[PeerType]) -> PeerType: ...
 
     @overload
-    async def get_peer(self, db: InfrahubDatabase, peer_type: Literal[None] = ...) -> Node: ...
+    async def get_peer(self, db: InfrahubDatabase, peer_type: None = ...) -> Node: ...
 
-    async def get_peer(self, db: InfrahubDatabase, peer_type: type[PeerType] | None = None) -> Any:  # pylint: disable=unused-argument
+    async def get_peer(self, db: InfrahubDatabase, peer_type: type[PeerType] | None = None) -> Any:  # noqa: ARG002
         """Return the peer of the relationship."""
         if self._peer is None:
             await self._get_peer(db=db)
@@ -326,7 +335,7 @@ class Relationship(FlagPropertyMixin, NodePropertyMixin):
 
         return different_properties
 
-    async def _create(self, db: InfrahubDatabase, at: Optional[Timestamp] = None) -> None:
+    async def _create(self, db: InfrahubDatabase, at: Timestamp | None = None) -> None:
         """Add a relationship with another object by creating a new relationship node."""
 
         create_at = Timestamp(at)
@@ -353,7 +362,7 @@ class Relationship(FlagPropertyMixin, NodePropertyMixin):
         db: InfrahubDatabase,
         properties_to_update: list[str],
         data: RelationshipPeerData,
-        at: Optional[Timestamp] = None,
+        at: Timestamp | None = None,
     ) -> None:
         """Update the properties of an existing relationship."""
 
@@ -381,7 +390,7 @@ class Relationship(FlagPropertyMixin, NodePropertyMixin):
         )
         await query.execute(db=db)
 
-    async def delete(self, db: InfrahubDatabase, at: Optional[Timestamp] = None) -> None:
+    async def delete(self, db: InfrahubDatabase, at: Timestamp | None = None) -> None:
         delete_at = Timestamp(at)
 
         node = await self.get_node(db=db)
@@ -465,7 +474,7 @@ class Relationship(FlagPropertyMixin, NodePropertyMixin):
             await self.set_peer(value=assigned_peer)
             self.set_source(value=pool.id)
 
-    async def save(self, db: InfrahubDatabase, at: Optional[Timestamp] = None) -> Self:
+    async def save(self, db: InfrahubDatabase, at: Timestamp | None = None) -> Self:
         """Create or Update the Relationship in the database."""
 
         save_at = Timestamp(at)
@@ -476,9 +485,7 @@ class Relationship(FlagPropertyMixin, NodePropertyMixin):
 
         return self
 
-    async def to_graphql(
-        self, fields: Optional[dict], db: InfrahubDatabase, related_node_ids: Optional[set] = None
-    ) -> dict:
+    async def to_graphql(self, fields: dict | None, db: InfrahubDatabase, related_node_ids: set | None = None) -> dict:
         """Generate GraphQL Payload for the associated Peer."""
 
         if not fields:
@@ -520,8 +527,6 @@ class Relationship(FlagPropertyMixin, NodePropertyMixin):
         return response
 
     async def get_create_data(self, db: InfrahubDatabase) -> RelationshipCreateData:
-        # pylint: disable=no-member
-
         branch = self.get_branch_based_on_support_type()
 
         await self.resolve(db=db)
@@ -717,8 +722,9 @@ class RelationshipManager:
             min_count=0 if self.schema.optional else self.schema.min_count,
             max_count=self.schema.max_count,
         )
-        self._relationship_id_details: Optional[RelationshipUpdateDetails] = None
+        self._relationship_id_details: RelationshipUpdateDetails | None = None
         self.has_fetched_relationships: bool = False
+        self.lock = asyncio.Lock()
 
     @classmethod
     async def init(
@@ -728,7 +734,7 @@ class RelationshipManager:
         branch: Branch,
         at: Timestamp,
         node: Node,
-        data: Optional[Union[dict, list, str]] = None,
+        data: dict | list | str | None = None,
     ) -> RelationshipManager:
         rm = cls(schema=schema, branch=branch, at=at, node=node)
 
@@ -747,7 +753,7 @@ class RelationshipManager:
         await rm._validate_hierarchy()
 
         for item in data:
-            if not isinstance(item, (rm.rel_class, str, dict)) and not hasattr(item, "_schema"):
+            if not isinstance(item, rm.rel_class | str | dict) and not hasattr(item, "_schema"):
                 raise ValidationError({rm.name: f"Invalid data provided to form a relationship {item}"})
 
             rel = rm.rel_class(schema=rm.schema, branch=rm.branch, at=rm.at, node=rm.node)
@@ -811,7 +817,7 @@ class RelationshipManager:
     async def get_peer(
         self,
         db: InfrahubDatabase,
-        peer_type: Literal[None] = ...,
+        peer_type: None = ...,
         raise_on_error: Literal[False] = ...,
     ) -> Node | None: ...
 
@@ -819,7 +825,7 @@ class RelationshipManager:
     async def get_peer(
         self,
         db: InfrahubDatabase,
-        peer_type: Literal[None] = ...,
+        peer_type: None = ...,
         raise_on_error: Literal[True] = ...,
     ) -> Node: ...
 
@@ -827,14 +833,14 @@ class RelationshipManager:
     async def get_peer(
         self,
         db: InfrahubDatabase,
-        peer_type: Literal[None] = ...,
+        peer_type: None = ...,
         raise_on_error: bool = ...,
     ) -> Node: ...
 
     async def get_peer(
         self,
         db: InfrahubDatabase,
-        peer_type: type[PeerType] | None = None,  # pylint: disable=unused-argument
+        peer_type: type[PeerType] | None = None,  # noqa: ARG002
         raise_on_error: bool = False,
     ) -> Node | PeerType | None:
         if self.schema.cardinality == "many":
@@ -861,14 +867,14 @@ class RelationshipManager:
     async def get_peers(
         self,
         db: InfrahubDatabase,
-        peer_type: Literal[None] = None,
+        peer_type: None = None,
         branch_agnostic: bool = ...,
     ) -> Mapping[str, Node]: ...
 
     async def get_peers(
         self,
         db: InfrahubDatabase,
-        peer_type: type[PeerType] | None = None,  # pylint: disable=unused-argument
+        peer_type: type[PeerType] | None = None,  # noqa: ARG002
         branch_agnostic: bool = False,
     ) -> Mapping[str, Node | PeerType]:
         rels = await self.get_relationships(db=db, branch_agnostic=branch_agnostic)
@@ -882,6 +888,8 @@ class RelationshipManager:
         """If the attribute is branch aware, return the Branch object associated with this attribute
         If the attribute is branch agnostic return the Global Branch
 
+        Note that if this relationship is Aware and source node is Agnostic, it will return -global- branch.
+
         Returns:
             Branch:
         """
@@ -892,7 +900,7 @@ class RelationshipManager:
     async def fetch_relationship_ids(
         self,
         db: InfrahubDatabase,
-        at: Optional[Timestamp] = None,
+        at: Timestamp | None = None,
         branch_agnostic: bool = False,
         force_refresh: bool = True,
     ) -> RelationshipUpdateDetails:
@@ -934,7 +942,7 @@ class RelationshipManager:
     async def _fetch_relationships(
         self,
         db: InfrahubDatabase,
-        at: Optional[Timestamp] = None,
+        at: Timestamp | None = None,
         branch_agnostic: bool = False,
         force_refresh: bool = True,
     ) -> None:
@@ -957,7 +965,7 @@ class RelationshipManager:
         self.has_fetched_relationships = True
 
         for peer_id in details.peer_ids_present_local_only:
-            await self.remove(peer_id=peer_id, db=db)
+            await self.remove_locally(peer_id=peer_id, db=db)
 
     async def get(self, db: InfrahubDatabase) -> Relationship | list[Relationship] | None:
         rels = await self.get_relationships(db=db)
@@ -969,20 +977,37 @@ class RelationshipManager:
 
         return rels
 
+    async def get_parent(
+        self, db: InfrahubDatabase, branch_agnostic: bool = False, force_refresh: bool = False
+    ) -> Relationship | None:
+        if self.schema.kind == RelationshipKind.PARENT:
+            for relationship in await self.get_relationships(
+                db=db, branch_agnostic=branch_agnostic, force_refresh=force_refresh
+            ):
+                # As parent relationships requires cardinality=one there will always only be one relationship
+                # here even though it's within a loop
+                return relationship
+
+        return None
+
     async def get_relationships(
         self, db: InfrahubDatabase, branch_agnostic: bool = False, force_refresh: bool = False
     ) -> list[Relationship]:
-        if force_refresh or not self.has_fetched_relationships:
-            await self._fetch_relationships(db=db, branch_agnostic=branch_agnostic, force_refresh=force_refresh)
+        # Use lock to avoid concurrent mutations on this object. This may typically happen while querying two nodes
+        # having the same parent, with the parent having another extra relationship. Concurrent coroutines will try to
+        # add this relationship to this object, and the second one will fail. See https://github.com/opsmill/infrahub/issues/5474.
+        # Note it also prevents extra relationships fetching. Ideally, DataLoader would fetch all needed data only once
+        # and exporting node to graphql would then not mutate this object.
+        async with self.lock:
+            if force_refresh or not self.has_fetched_relationships:
+                await self._fetch_relationships(db=db, branch_agnostic=branch_agnostic, force_refresh=force_refresh)
 
         return self._relationships.as_list()
 
-    async def update(  # pylint: disable=too-many-branches
-        self, data: Union[list[Union[str, Node]], dict[str, Any], str, Node, None], db: InfrahubDatabase
-    ) -> bool:
+    async def update(self, data: list[str | Node] | dict[str, Any] | str | Node | None, db: InfrahubDatabase) -> bool:
         """Replace and Update the list of relationships with this one."""
         if not isinstance(data, list):
-            list_data: Sequence[Union[str, Node, dict[str, Any], None]] = [data]
+            list_data: Sequence[str | Node | dict[str, Any] | None] = [data]
         else:
             list_data = data
 
@@ -994,7 +1019,7 @@ class RelationshipManager:
         changed = False
 
         for item in list_data:
-            if not isinstance(item, (self.rel_class, str, dict, type(None))) and not hasattr(item, "_schema"):
+            if not isinstance(item, self.rel_class | str | dict | type(None)) and not hasattr(item, "_schema"):
                 raise ValidationError({self.name: f"Invalid data provided to form a relationship {item}"})
 
             if hasattr(item, "_schema"):
@@ -1032,7 +1057,7 @@ class RelationshipManager:
             changed = True
 
         # Check if some relationship got removed by checking if the previous list of relationship is a subset of the current list of not
-        if set(list(previous_relationships.keys())) <= {rel.peer_id for rel in await self.get_relationships(db=db)}:
+        if set(previous_relationships.keys()) <= {rel.peer_id for rel in await self.get_relationships(db=db)}:
             changed = True
 
         if changed:
@@ -1040,9 +1065,9 @@ class RelationshipManager:
 
         return changed
 
-    async def add(self, data: Union[dict[str, Any], Node], db: InfrahubDatabase) -> bool:
+    async def add(self, data: dict[str, Any] | Node, db: InfrahubDatabase) -> bool:
         """Add a new relationship to the list of existing ones, avoid duplication."""
-        if not isinstance(data, (self.rel_class, dict)) and not hasattr(data, "_schema"):
+        if not isinstance(data, self.rel_class | dict) and not hasattr(data, "_schema"):
             raise ValidationError({self.name: f"Invalid data provided to form a relationship {data}"})
 
         await self._validate_hierarchy()
@@ -1069,22 +1094,17 @@ class RelationshipManager:
         for rel in self._relationships:
             await rel.resolve(db=db)
 
-    async def remove(
+    async def remove_locally(
         self,
-        peer_id: Union[str, UUID],
+        peer_id: str | UUID,
         db: InfrahubDatabase,
-        update_db: bool = False,
     ) -> bool:
-        """Remove a peer id from the local relationships list,
-        need to investigate if and when we should update the relationship in the database."""
+        """Remove a peer id from the local relationships list"""
 
         for idx, rel in enumerate(await self.get_relationships(db=db)):
             if str(rel.peer_id) != str(peer_id):
                 continue
 
-            if update_db:
-                await rel.delete(db=db)
-
             self._relationships.pop(idx)
             return True
 
@@ -1094,21 +1114,20 @@ class RelationshipManager:
         self,
         db: InfrahubDatabase,
         peer_data: RelationshipPeerData,
-        at: Optional[Timestamp] = None,
+        at: Timestamp | None = None,
     ) -> None:
         remove_at = Timestamp(at)
         branch = self.get_branch_based_on_support_type()
 
         # - Update the existing relationship if we are on the same branch
         rel_ids_per_branch = peer_data.rel_ids_per_branch()
+
+        # In which cases do we end up here and do not want to set `to` time?
         if branch.name in rel_ids_per_branch:
-            await update_relationships_to([str(ri) for ri in rel_ids_per_branch[self.branch.name]], to=remove_at, db=db)
+            await update_relationships_to([str(ri) for ri in rel_ids_per_branch[branch.name]], to=remove_at, db=db)
 
         # - Create a new rel of type DELETED if the existing relationship is on a different branch
-        rel_branches: set[str] = set()
-        if peer_data.rels:
-            rel_branches = {r.branch for r in peer_data.rels}
-        if rel_branches == {peer_data.branch}:
+        if peer_data.rels and {r.branch for r in peer_data.rels} == {peer_data.branch}:
             return
 
         query = await RelationshipDataDeleteQuery.init(
@@ -1122,18 +1141,22 @@ class RelationshipManager:
         )
         await query.execute(db=db)
 
-    async def save(self, db: InfrahubDatabase, at: Optional[Timestamp] = None) -> Self:
+    async def save(
+        self, db: InfrahubDatabase, at: Timestamp | None = None
+    ) -> RelationshipCardinalityManyChangelog | RelationshipCardinalityOneChangelog:
         """Create or Update the Relationship in the database."""
 
         await self.resolve(db=db)
 
         save_at = Timestamp(at)
         details = await self.fetch_relationship_ids(db=db, force_refresh=True)
+        relationship_mapper = ChangelogRelationshipMapper(schema=self.schema)
 
         # If we have previously fetched the relationships from the database
         # Update the one in the database that shouldn't be here.
         if self.has_fetched_relationships:
             for peer_id in details.peer_ids_present_database_only:
+                relationship_mapper.remove_peer(peer_data=details.peers_database[peer_id])
                 await self.remove_in_db(peer_data=details.peers_database[peer_id], at=save_at, db=db)
 
         # Create the new relationship that are not present in the database
@@ -1142,6 +1165,8 @@ class RelationshipManager:
             if rel.peer_id in details.peer_ids_present_local_only:
                 await rel.save(at=save_at, db=db)
 
+                relationship_mapper.add_peer_from_relationship(relationship=rel)
+
             elif rel.peer_id in details.peer_ids_present_both:
                 if properties_not_matching := rel.compare_properties_with_data(
                     data=details.peers_database[rel.peer_id]
@@ -1152,22 +1177,37 @@ class RelationshipManager:
                         data=details.peers_database[rel.peer_id],
                         db=db,
                     )
+                    relationship_mapper.add_updated_relationship(
+                        relationship=rel,
+                        old_data=details.peers_database[rel.peer_id],
+                        properties_to_update=properties_not_matching,
+                    )
+            elif rel.schema.kind == RelationshipKind.PARENT:
+                relationship_mapper.add_parent_from_relationship(relationship=rel)
 
-        return self
+        return relationship_mapper.changelog
 
-    async def delete(self, db: InfrahubDatabase, at: Optional[Timestamp] = None) -> None:
+    async def delete(
+        self, db: InfrahubDatabase, at: Timestamp | None = None
+    ) -> RelationshipCardinalityManyChangelog | RelationshipCardinalityOneChangelog:
         """Delete all the relationships."""
 
         delete_at = Timestamp(at)
+        relationship_mapper = ChangelogRelationshipMapper(schema=self.schema)
 
         await self._fetch_relationships(at=delete_at, db=db, force_refresh=True)
 
         for rel in await self.get_relationships(db=db):
+            relationship_mapper.delete_relationship(
+                peer_kind=rel.get_peer_kind(), peer_id=rel.get_peer_id(), rel_schema=rel.schema
+            )
             await rel.delete(at=delete_at, db=db)
 
+        return relationship_mapper.changelog
+
     async def to_graphql(
-        self, db: InfrahubDatabase, fields: Optional[dict] = None, related_node_ids: Optional[set] = None
-    ) -> Union[dict, None]:
+        self, db: InfrahubDatabase, fields: dict | None = None, related_node_ids: set | None = None
+    ) -> dict | None:
         # NOTE Need to investigate when and why we are passing the peer directly here, how do we account for many relationship
         if self.schema.cardinality == "many":
             raise TypeError("to_graphql is not available for relationship with multiple cardinality")
@@ -1180,7 +1220,7 @@ class RelationshipManager:
 
     async def _validate_hierarchy(self) -> None:
         schema = self.node.get_schema()
-        if schema.is_profile_schema or not schema.hierarchy:  # type: ignore[union-attr]
+        if schema.is_profile_schema or schema.is_template_schema or not schema.hierarchy:  # type: ignore[union-attr]
             return
 
         if self.name == "parent" and not schema.parent:  # type: ignore[union-attr]
