@@ -1,5 +1,3 @@
-from typing import Union
-
 from pydantic import BaseModel, Field
 
 from infrahub.core.branch import Branch
@@ -10,7 +8,7 @@ from infrahub.core.schema import GenericSchema, NodeSchema
 class SchemaConstraintValidatorRequest(BaseModel):
     branch: Branch = Field(..., description="The name of the branch to target")
     constraint_name: str = Field(..., description="The name of the constraint to validate")
-    node_schema: Union[NodeSchema, GenericSchema] = Field(..., description="Schema of Node or Generic to validate")
+    node_schema: NodeSchema | GenericSchema = Field(..., description="Schema of Node or Generic to validate")
     schema_path: SchemaPath = Field(..., description="SchemaPath to the element of the schema to validate")
 
 
