@@ -3,7 +3,7 @@ from __future__ import annotations
 import shutil
 from abc import ABC, abstractmethod
 from pathlib import Path
-from typing import TYPE_CHECKING, NoReturn, Optional, Union
+from typing import TYPE_CHECKING, NoReturn
 from uuid import UUID  # noqa: TC003
 
 import git
@@ -12,6 +12,7 @@ from git.exc import GitCommandError, InvalidGitRepositoryError
 from git.refs.remote import RemoteReference
 from infrahub_sdk import InfrahubClient  # noqa: TC002
 from prefect import Flow, Task
+from prefect.logging import get_run_logger
 from pydantic import BaseModel, ConfigDict, Field
 from pydantic import ValidationError as PydanticValidationError
 
@@ -121,7 +122,7 @@ class BranchInLocal(BaseModel):
     has_worktree: bool = False
 
 
-class InfrahubRepositoryBase(BaseModel, ABC):  # pylint: disable=too-many-public-methods
+class InfrahubRepositoryBase(BaseModel, ABC):
     """
     Local version of a Git repository organized to work with Infrahub.
     The idea is that all commits that are being tracked in the graph will be checkout out
@@ -143,12 +144,12 @@ class InfrahubRepositoryBase(BaseModel, ABC):  # pylint: disable=too-many-public
         False, description="Flag to indicate if a remote repository (named origin) is present in the config."
     )
 
-    client: Optional[InfrahubClient] = Field(
+    client: InfrahubClient | None = Field(
         default=None,
         description="Infrahub Client, used to query the Repository and Branch information in the graph and to update the commit.",
     )
 
-    cache_repo: Optional[Repo] = Field(None, description="Internal cache of the GitPython Repo object")
+    cache_repo: Repo | None = Field(None, description="Internal cache of the GitPython Repo object")
     service: InfrahubServices = Field(
         ..., description="Service object with access to the message queue, the database etc.."
     )
@@ -394,9 +395,7 @@ class InfrahubRepositoryBase(BaseModel, ABC):  # pylint: disable=too-many-public
         return [Worktree.init(response) for response in responses]
 
     def get_client(self) -> InfrahubClient:
-        if self.client:
-            return self.client
-        return self.service.client
+        return self.sdk
 
     def get_location(self) -> str:
         if self.location:
@@ -478,6 +477,36 @@ class InfrahubRepositoryBase(BaseModel, ABC):  # pylint: disable=too-many-public
     def get_commit_value(self, branch_name: str, remote: bool = False) -> str:
         raise NotImplementedError()
 
+    def has_conflicting_changes(self, target_branch: str, source_branch: str) -> bool:
+        """Use merge tree to spot conflicts and tell if there is any."""
+        repo = self.get_git_repo_main()
+
+        if repo.remotes:
+            # Ensure we have the latest changes from the remote
+            info = repo.remotes.origin.fetch(source_branch)
+
+            target = repo.branches[target_branch]
+            source = repo.commit(info[0].ref)
+
+            merge_base = repo.merge_base(target.commit, source)[0]
+            merge_tree_output = repo.git.merge_tree(merge_base.hexsha, target.commit.hexsha, source.hexsha)
+        else:
+            target = repo.branches[target_branch]
+            source = repo.branches[source_branch]
+
+            merge_base = repo.merge_base(target.commit, source)[0]
+            merge_tree_output = repo.git.merge_tree(merge_base.hexsha, target.commit.hexsha, source.commit.hexsha)
+
+        log.debug(
+            f"Merging {source_branch} into {target_branch} will bring changes",
+            repository=self.name,
+            source=source_branch,
+            target=target_branch,
+            merge_structure=merge_tree_output,
+        )
+
+        return any(marker in merge_tree_output for marker in ("<<<<<<<", "=======", ">>>>>>>"))
+
     async def update_commit_value(self, branch_name: str, commit: str) -> bool:
         """Compare the value of the commit in the graph with the current commit on the filesystem.
         update it if they don't match.
@@ -540,7 +569,10 @@ class InfrahubRepositoryBase(BaseModel, ABC):  # pylint: disable=too-many-public
         if remote_branch:
             br_repo = self.get_git_repo_worktree(identifier=branch_name)
             br_repo.head.reference.set_tracking_branch(remote_branch[0])
-            br_repo.remotes.origin.pull(branch_name)
+            try:
+                br_repo.remotes.origin.pull(branch_name)
+            except GitCommandError as exc:
+                self._raise_enriched_error(error=exc, branch_name=branch_name)
             self.create_commit_worktree(str(br_repo.head.reference.commit))
             log.debug(
                 f"Branch {branch_name} created in Git, tracking remote branch {remote_branch[0]}.",
@@ -552,7 +584,7 @@ class InfrahubRepositoryBase(BaseModel, ABC):  # pylint: disable=too-many-public
 
         return True
 
-    def create_commit_worktree(self, commit: str) -> Union[bool, Worktree]:
+    def create_commit_worktree(self, commit: str) -> bool | Worktree:
         """Create a new worktree for a given commit."""
 
         # Check of the worktree already exist
@@ -668,14 +700,14 @@ class InfrahubRepositoryBase(BaseModel, ABC):  # pylint: disable=too-many-public
                 log.info("New commit detected", repository=self.name, branch=branch_name)
                 updated_branches.append(branch_name)
 
-        return sorted(list(new_branches)), sorted(updated_branches)
+        return sorted(new_branches), sorted(updated_branches)
 
-    async def validate_remote_branch(self, branch_name: str) -> bool:
-        """Validate a branch present on the remote repository.
-        To check a branch we need to first create a worktree in the temporary folder then apply some checks:
-        - xxx
+    def validate_remote_branch(self, branch_name: str) -> bool:
+        """Process a remote branch to validate that we can use it safely.
 
-        At the end, we need to delete the worktree in the temporary folder.
+        - Make sure that the branch name won't conflict with infrahub's default branch
+        - Make sure that a representation if the branch can be created in the database
+        - Make sure that there are no conflicts that would prevent it from being merged
         """
         if branch_name == registry.default_branch and branch_name != self.default_branch:
             # If the default branch of Infrahub and the git repository differs we map the repository
@@ -683,6 +715,7 @@ class InfrahubRepositoryBase(BaseModel, ABC):  # pylint: disable=too-many-public
             # repository if it matches the default branch of Infrahub
             log.warning("Ignoring import of mismatched default branch", branch=branch_name, repository=self.name)
             return False
+
         try:
             # Check if the branch can be created in the database
             Branch(name=branch_name)
@@ -692,6 +725,13 @@ class InfrahubRepositoryBase(BaseModel, ABC):  # pylint: disable=too-many-public
             )
             return False
 
+        # Make sure the branch won't conflict on merge
+        if self.has_conflicting_changes(target_branch=self.default_branch, source_branch=branch_name):
+            get_run_logger().warning(
+                f"Remote branch {branch_name} will cause conflicts, they need to be resolved before importing the branch into Infrahub"
+            )
+            return False
+
         # Find the commit on the remote branch
         # Check out the commit in a worktree
         # Validate
@@ -704,7 +744,7 @@ class InfrahubRepositoryBase(BaseModel, ABC):  # pylint: disable=too-many-public
         branch_id: str | None = None,
         create_if_missing: bool = False,
         update_commit_value: bool = True,
-    ) -> Union[bool, str]:
+    ) -> bool | str:
         """Pull the latest update from the remote repository on a given branch."""
 
         if not self.has_origin:
@@ -772,10 +812,10 @@ class InfrahubRepositoryBase(BaseModel, ABC):  # pylint: disable=too-many-public
 
     async def find_files(
         self,
-        extension: Union[str, list[str]],
+        extension: str | list[str],
         branch_name: str | None = None,
         commit: str | None = None,
-        directory: Optional[Path] = None,
+        directory: Path | None = None,
     ) -> list[Path]:
         """Return the path of all files matching a specific extension in a given Branch or Commit."""
         if not branch_name and not commit:
@@ -855,10 +895,10 @@ class InfrahubRepositoryBase(BaseModel, ABC):  # pylint: disable=too-many-public
                 message=f"Authentication failed for {name}, please validate the credentials.",
             ) from error
 
-        if "Need to specify how to reconcile" in error.stderr:
+        if any(err in error.stderr for err in ("Need to specify how to reconcile", "because you have unmerged files")):
             raise RepositoryError(
                 identifier=name,
-                message=f"Unable to pull the branch {branch_name} for repository {name}, there is a conflict that must be resolved.",
+                message=f"Unable to pull the branch {branch_name} for repository {name}, there are conflicts that must be resolved.",
             ) from error
 
         if "fatal: could not read Username for" in error.stderr and "terminal prompts disable" in error.stderr:
