@@ -3,7 +3,7 @@ from __future__ import annotations
 import hashlib
 import importlib
 import sys
-from typing import TYPE_CHECKING, Any, Optional, Union
+from typing import TYPE_CHECKING, Any
 
 import jinja2
 import ujson
@@ -37,11 +37,14 @@ from pydantic import BaseModel, Field
 from pydantic import ValidationError as PydanticValidationError
 from typing_extensions import Self
 
-from infrahub.core.constants import InfrahubKind, RepositorySyncStatus
+from infrahub.core.constants import ArtifactStatus, ContentType, InfrahubKind, RepositorySyncStatus
+from infrahub.core.registry import registry
+from infrahub.events.artifact_action import ArtifactCreatedEvent, ArtifactUpdatedEvent
+from infrahub.events.models import EventMeta
 from infrahub.events.repository_action import CommitUpdatedEvent
 from infrahub.exceptions import CheckError, RepositoryInvalidFileSystemError, TransformError
 from infrahub.git.base import InfrahubRepositoryBase, extract_repo_file_information
-from infrahub.services import InfrahubServices
+from infrahub.log import get_logger
 from infrahub.workflows.utils import add_tags
 
 if TYPE_CHECKING:
@@ -49,13 +52,12 @@ if TYPE_CHECKING:
 
     from infrahub_sdk.checks import InfrahubCheck
     from infrahub_sdk.node import InfrahubNode
-    from infrahub_sdk.schema import InfrahubRepositoryArtifactDefinitionConfig
+    from infrahub_sdk.schema.repository import InfrahubRepositoryArtifactDefinitionConfig
     from infrahub_sdk.transforms import InfrahubTransform
 
+    from infrahub.artifacts.models import CheckArtifactCreate
     from infrahub.git.models import RequestArtifactGenerate
-    from infrahub.message_bus import messages
-
-# pylint: disable=too-many-lines
+    from infrahub.services import InfrahubServices
 
 
 class ArtifactGenerateResult(BaseModel):
@@ -91,10 +93,10 @@ class CheckDefinitionInformation(BaseModel):
     timeout: int
     """Timeout for the Check."""
 
-    parameters: Optional[dict] = None
+    parameters: dict | None = None
     """Additional Parameters to extract from each target (if targets is provided)"""
 
-    targets: Optional[str] = Field(default=None, description="Targets if not a global check")
+    targets: str | None = Field(default=None, description="Targets if not a global check")
 
 
 class TransformPythonInformation(BaseModel):
@@ -120,7 +122,7 @@ class TransformPythonInformation(BaseModel):
     """Timeout for the function."""
 
 
-class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=too-many-public-methods
+class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):
     """
     This class provides interfaces to read and process information from .infrahub.yml files and can perform
     actions for objects defined within those files.
@@ -130,20 +132,20 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
     """
 
     @classmethod
-    async def init(cls, commit: str | None = None, service: InfrahubServices | None = None, **kwargs: Any) -> Self:
-        service = service or InfrahubServices()
+    async def init(cls, service: InfrahubServices, commit: str | None = None, **kwargs: Any) -> Self:
         self = cls(service=service, **kwargs)
+        log = get_logger()
         try:
             self.validate_local_directories()
         except RepositoryInvalidFileSystemError:
             await self.ensure_location_is_defined()
             await self.create_locally(infrahub_branch_name=self.infrahub_branch_name, update_commit_value=False)
-            service.log.info(f"Initialized the local directory for {self.name} because it was missing.")
+            log.info(f"Initialized the local directory for {self.name} because it was missing.")
 
         if commit:
             self.get_commit_worktree(commit=commit)
 
-        service.log.debug(
+        log.debug(
             f"Initiated the object on an existing directory for {self.name}",
         )
         return self
@@ -159,7 +161,7 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
 
     @flow(name="import-object-from-file", flow_run_name="Import objects")
     async def import_objects_from_files(
-        self, infrahub_branch_name: str, git_branch_name: Optional[str] = None, commit: Optional[str] = None
+        self, infrahub_branch_name: str, git_branch_name: str | None = None, commit: str | None = None
     ) -> None:
         if not commit:
             commit = self.get_commit_value(branch_name=git_branch_name or infrahub_branch_name)
@@ -169,29 +171,29 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
         self.create_commit_worktree(commit)
         await self._update_sync_status(branch_name=infrahub_branch_name, status=RepositorySyncStatus.SYNCING)
 
-        config_file = await self.get_repository_config(branch_name=infrahub_branch_name, commit=commit)
+        config_file = await self.get_repository_config(branch_name=infrahub_branch_name, commit=commit)  # type: ignore[misc]
         sync_status = RepositorySyncStatus.IN_SYNC if config_file else RepositorySyncStatus.ERROR_IMPORT
         error: Exception | None = None
 
         try:
             if config_file:
-                await self.import_schema_files(branch_name=infrahub_branch_name, commit=commit, config_file=config_file)
+                await self.import_schema_files(branch_name=infrahub_branch_name, commit=commit, config_file=config_file)  # type: ignore[misc]
 
                 await self.import_all_graphql_query(
                     branch_name=infrahub_branch_name, commit=commit, config_file=config_file
-                )
+                )  # type: ignore[misc]
 
-                await self.import_all_python_files(
+                await self.import_all_python_files(  # type: ignore[call-overload]
                     branch_name=infrahub_branch_name, commit=commit, config_file=config_file
-                )
+                )  # type: ignore[misc]
                 await self.import_jinja2_transforms(
                     branch_name=infrahub_branch_name, commit=commit, config_file=config_file
-                )
+                )  # type: ignore[misc]
                 await self.import_artifact_definitions(
                     branch_name=infrahub_branch_name, commit=commit, config_file=config_file
-                )
+                )  # type: ignore[misc]
 
-        except Exception as exc:  # pylint: disable=broad-exception-caught
+        except Exception as exc:
             sync_status = RepositorySyncStatus.ERROR_IMPORT
             error = exc
 
@@ -200,9 +202,13 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
         if error:
             raise error
 
+        infrahub_branch = registry.get_branch_from_registry(branch=infrahub_branch_name)
         await self.service.event.send(
             CommitUpdatedEvent(
-                branch=infrahub_branch_name, commit=commit, repository_name=self.name, repository_id=str(self.id)
+                commit=commit,
+                repository_name=self.name,
+                repository_id=str(self.id),
+                meta=EventMeta.with_dummy_context(branch=infrahub_branch),
             )
         )
 
@@ -230,11 +236,11 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
             tracker="mutation-repository-update-admin-status",
         )
 
-    @task(name="import-jinja2-tansforms", task_run_name="Import Jinja2 transform", cache_policy=NONE)
+    @task(name="import-jinja2-tansforms", task_run_name="Import Jinja2 transform", cache_policy=NONE)  # type: ignore[arg-type]
     async def import_jinja2_transforms(
         self,
         branch_name: str,
-        commit: str,  # pylint: disable=unused-argument
+        commit: str,  # noqa: ARG002
         config_file: InfrahubRepositoryConfig,
     ) -> None:
         log = get_run_logger()
@@ -312,7 +318,6 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
     async def compare_jinja2_transform(
         cls, existing_transform: CoreTransformJinja2, local_transform: InfrahubRepositoryJinja2
     ) -> bool:
-        # pylint: disable=no-member
         if (
             existing_transform.description.value != local_transform.description
             or existing_transform.template_path.value != local_transform.template_path
@@ -325,7 +330,6 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
     async def update_jinja2_transform(
         self, existing_transform: CoreTransformJinja2, local_transform: InfrahubRepositoryJinja2
     ) -> None:
-        # pylint: disable=no-member
         if existing_transform.description.value != local_transform.description:
             existing_transform.description.value = local_transform.description
 
@@ -337,11 +341,11 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
 
         await existing_transform.save()
 
-    @task(name="import-artifact-definitions", task_run_name="Import Artifact Definitions", cache_policy=NONE)
+    @task(name="import-artifact-definitions", task_run_name="Import Artifact Definitions", cache_policy=NONE)  # type: ignore[arg-type]
     async def import_artifact_definitions(
         self,
         branch_name: str,
-        commit: str,  # pylint: disable=unused-argument
+        commit: str,  # noqa: ARG002
         config_file: InfrahubRepositoryConfig,
     ) -> None:
         log = get_run_logger()
@@ -438,8 +442,8 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
 
         await existing_artifact_definition.save()
 
-    @task(name="repository-get-config", task_run_name="get repository config", cache_policy=NONE)
-    async def get_repository_config(self, branch_name: str, commit: str) -> Optional[InfrahubRepositoryConfig]:
+    @task(name="repository-get-config", task_run_name="get repository config", cache_policy=NONE)  # type: ignore[arg-type]
+    async def get_repository_config(self, branch_name: str, commit: str) -> InfrahubRepositoryConfig | None:
         branch_wt = self.get_worktree(identifier=commit or branch_name)
         log = get_run_logger()
 
@@ -467,9 +471,8 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
             log.error(f"Unable to load the configuration file {config_file_name}, the format is not valid  : {exc}")
             return None
 
-    @task(name="import-schema-files", task_run_name="Import schema files", cache_policy=NONE)
+    @task(name="import-schema-files", task_run_name="Import schema files", cache_policy=NONE)  # type: ignore[arg-type]
     async def import_schema_files(self, branch_name: str, commit: str, config_file: InfrahubRepositoryConfig) -> None:
-        # pylint: disable=too-many-branches
         log = get_run_logger()
         branch_wt = self.get_worktree(identifier=commit or branch_name)
 
@@ -540,7 +543,7 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
         for schema_file in schemas_data:
             log.info(f"schema '{schema_file.identifier}' loaded successfully!")
 
-    @task(name="import-graphql-queries", task_run_name="Import GraphQL Queries", cache_policy=NONE)
+    @task(name="import-graphql-queries", task_run_name="Import GraphQL Queries", cache_policy=NONE)  # type: ignore[arg-type]
     async def import_all_graphql_query(
         self, branch_name: str, commit: str, config_file: InfrahubRepositoryConfig
     ) -> None:
@@ -598,7 +601,7 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
         await obj.save()
         return obj
 
-    @task(name="import-python-check-definitions", task_run_name="Import Python Check Definitions", cache_policy=NONE)
+    @task(name="import-python-check-definitions", task_run_name="Import Python Check Definitions", cache_policy=NONE)  # type: ignore[arg-type]
     async def import_python_check_definitions(
         self, branch_name: str, commit: str, config_file: InfrahubRepositoryConfig
     ) -> None:
@@ -633,7 +636,7 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
                     module=module,
                     file_path=file_info.relative_path_file,
                     check_definition=check,
-                )
+                )  # type: ignore[misc]
             )
 
         local_check_definitions = {check.name: check for check in checks}
@@ -669,7 +672,7 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
             log.info(f"CheckDefinition '{check_name!r}' not found locally, deleting")
             await check_definition_in_graph[check_name].delete()
 
-    @task(name="import-generator-definitions", task_run_name="Import Generator Definitions", cache_policy=NONE)
+    @task(name="import-generator-definitions", task_run_name="Import Generator Definitions", cache_policy=NONE)  # type: ignore[arg-type]
     async def import_generator_definitions(
         self, branch_name: str, commit: str, config_file: InfrahubRepositoryConfig
     ) -> None:
@@ -748,7 +751,7 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
         if targets:
             generator.targets = targets[0].id
 
-        if (  # pylint: disable=too-many-boolean-expressions
+        if (
             existing_generator.query.id != generator.query
             or existing_generator.file_path.value != str(generator.file_path)
             or existing_generator.class_name.value != generator.class_name
@@ -759,7 +762,7 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
             return True
         return False
 
-    @task(name="import-python-transforms", task_run_name="Import Python Transforms", cache_policy=NONE)
+    @task(name="import-python-transforms", task_run_name="Import Python Transforms", cache_policy=NONE)  # type: ignore[arg-type]
     async def import_python_transforms(
         self, branch_name: str, commit: str, config_file: InfrahubRepositoryConfig
     ) -> None:
@@ -771,7 +774,7 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
         if str(self.directory_root) not in sys.path:
             sys.path.append(str(self.directory_root))
 
-        transforms = []
+        transforms: list[TransformPythonInformation] = []
         log.info(f"Found {len(config_file.python_transforms)} Python transforms in the repository")
 
         for transform in config_file.python_transforms:
@@ -794,14 +797,14 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
                     module=module,
                     file_path=file_info.relative_path_file,
                     transform=transform,
-                )
+                )  # type: ignore[misc]
             )
 
         local_transform_definitions = {transform.name: transform for transform in transforms}
         transform_definition_in_graph = {
             transform.name.value: transform
             for transform in await self.sdk.filters(
-                kind=InfrahubKind.TRANSFORMPYTHON, branch=branch_name, repository__ids=[str(self.id)]
+                kind=CoreTransformPython, branch=branch_name, repository__ids=[str(self.id)]
             )
         }
 
@@ -830,7 +833,7 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
             log.info(f"TransformPython {transform_name!r} not found locally, deleting")
             await transform_definition_in_graph[transform_name].delete()
 
-    @task(name="check-definition-get", task_run_name="Get Check Definition", cache_policy=NONE)
+    @task(name="check-definition-get", task_run_name="Get Check Definition", cache_policy=NONE)  # type: ignore[arg-type]
     async def get_check_definition(
         self,
         branch_name: str,
@@ -863,14 +866,14 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
                 )
             )
 
-        except Exception as exc:  # pylint: disable=broad-exception-caught
+        except Exception as exc:
             log.error(
                 f"An error occurred while processing the CheckDefinition {check_class.__name__} from {file_path} : {exc} "
             )
             raise
         return checks
 
-    @task(name="python-transform-get", task_run_name="Get Python Transform", cache_policy=NONE)
+    @task(name="python-transform-get", task_run_name="Get Python Transform", cache_policy=NONE)  # type: ignore[arg-type]
     async def get_python_transforms(
         self, branch_name: str, module: types.ModuleType, file_path: str, transform: InfrahubPythonTransformConfig
     ) -> list[TransformPythonInformation]:
@@ -896,7 +899,7 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
                 )
             )
 
-        except Exception as exc:  # pylint: disable=broad-exception-caught
+        except Exception as exc:
             log.error(
                 f"An error occurred while processing the PythonTransform {transform.name} from {file_path} : {exc} "
             )
@@ -1003,7 +1006,6 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
     ) -> bool:
         """Compare an existing Python Check Object with a Check Class
         and identify if we need to update the object in the database."""
-        # pylint: disable=too-many-boolean-expressions
         if (
             existing_check.query.id != check.query
             or existing_check.file_path.value != check.file_path
@@ -1029,7 +1031,7 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
         create_payload = self.sdk.schema.generate_payload_create(
             schema=schema,
             data=data,
-            source=self.id,
+            source=str(self.id),
             is_protected=True,
         )
         obj = await self.sdk.create(kind=CoreTransformPython, branch=branch_name, **create_payload)
@@ -1068,11 +1070,11 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
     ) -> None:
         await add_tags(branches=[branch_name], nodes=[str(self.id)])
 
-        await self.import_python_check_definitions(branch_name=branch_name, commit=commit, config_file=config_file)
-        await self.import_python_transforms(branch_name=branch_name, commit=commit, config_file=config_file)
-        await self.import_generator_definitions(branch_name=branch_name, commit=commit, config_file=config_file)
+        await self.import_python_check_definitions(branch_name=branch_name, commit=commit, config_file=config_file)  # type: ignore[misc]
+        await self.import_python_transforms(branch_name=branch_name, commit=commit, config_file=config_file)  # type: ignore[misc]
+        await self.import_generator_definitions(branch_name=branch_name, commit=commit, config_file=config_file)  # type: ignore[misc]
 
-    @task(name="jinja2-template-render", task_run_name="Render Jinja2 template", cache_policy=NONE)
+    @task(name="jinja2-template-render", task_run_name="Render Jinja2 template", cache_policy=NONE)  # type: ignore[arg-type]
     async def render_jinja2_template(self, commit: str, location: str, data: dict) -> str:
         log = get_run_logger()
         commit_worktree = self.get_commit_worktree(commit=commit)
@@ -1088,7 +1090,7 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
             log.error(str(exc), exc_info=True)
             raise TransformError(repository_name=self.name, commit=commit, location=location, message=str(exc)) from exc
 
-    @task(name="python-check-execute", task_run_name="Execute Python Check", cache_policy=NONE)
+    @task(name="python-check-execute", task_run_name="Execute Python Check", cache_policy=NONE)  # type: ignore[arg-type]
     async def execute_python_check(
         self,
         branch_name: str,
@@ -1096,7 +1098,7 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
         location: str,
         class_name: str,
         client: InfrahubClient,
-        params: Optional[dict] = None,
+        params: dict | None = None,
     ) -> InfrahubCheck:
         """Execute A Python Check stored in the repository."""
         log = get_run_logger()
@@ -1147,9 +1149,9 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
                 repository_name=self.name, class_name=class_name, commit=commit, location=location, message=str(exc)
             ) from exc
 
-    @task(name="python-transform-execute", task_run_name="Execute Python Transform", cache_policy=NONE)
+    @task(name="python-transform-execute", task_run_name="Execute Python Transform", cache_policy=NONE)  # type: ignore[arg-type]
     async def execute_python_transform(
-        self, branch_name: str, commit: str, location: str, client: InfrahubClient, data: Optional[dict] = None
+        self, branch_name: str, commit: str, location: str, client: InfrahubClient, data: dict | None = None
     ) -> Any:
         """Execute A Python Transform stored in the repository."""
         log = get_run_logger()
@@ -1210,6 +1212,7 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
         transformation: CoreTransformation,
         query: CoreGraphQLQuery,
     ) -> ArtifactGenerateResult:
+        """It doesn't look like this is used anywhere today ... we should either remove it or refactor render_artifact below to use this."""
         variables = target.extract(params=definition.parameters.value)
         response = await self.sdk.query_gql_query(
             name=query.name.value,
@@ -1222,23 +1225,27 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
         )
 
         if transformation.typename == InfrahubKind.TRANSFORMJINJA2:
-            artifact_content = await self.render_jinja2_template(
-                commit=commit, location=transformation.template_path.value, data=response
-            )
+            artifact_content = await self.render_jinja2_template.with_options(
+                timeout_seconds=transformation.timeout.value
+            )(commit=commit, location=transformation.template_path.value, data=response)  # type: ignore[misc]
         elif transformation.typename == InfrahubKind.TRANSFORMPYTHON:
             transformation_location = f"{transformation.file_path.value}::{transformation.class_name.value}"
-            artifact_content = await self.execute_python_transform(
+            artifact_content = await self.execute_python_transform.with_options(
+                timeout_seconds=transformation.timeout.value
+            )(
                 branch_name=branch_name,
                 commit=commit,
                 location=transformation_location,
                 data=response,
                 client=self.sdk,
-            )
+            )  # type: ignore[misc]
 
-        if definition.content_type.value == "application/json":
+        if definition.content_type.value == ContentType.APPLICATION_JSON.value and isinstance(artifact_content, dict):
             artifact_content_str = ujson.dumps(artifact_content, indent=2)
-        elif definition.content_type.value == "text/plain":
-            artifact_content_str = artifact_content
+        elif definition.content_type.value == ContentType.APPLICATION_YAML.value and isinstance(artifact_content, dict):
+            artifact_content_str = yaml.dump(artifact_content, indent=2)
+        else:
+            artifact_content_str = str(artifact_content)
 
         checksum = hashlib.md5(bytes(artifact_content_str, encoding="utf-8"), usedforsecurity=False).hexdigest()
 
@@ -1252,13 +1259,18 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
 
         artifact.checksum.value = checksum
         artifact.storage_id.value = storage_id
-        artifact.status.value = "Ready"
+        artifact.status.value = ArtifactStatus.READY.value
+        if artifact.name.value != definition.artifact_name.value:
+            artifact.name.value = definition.artifact_name.value
         await artifact.save()
 
         return ArtifactGenerateResult(changed=True, checksum=checksum, storage_id=storage_id, artifact_id=artifact.id)
 
     async def render_artifact(
-        self, artifact: CoreArtifact, message: Union[messages.CheckArtifactCreate, RequestArtifactGenerate]
+        self,
+        artifact: CoreArtifact,
+        artifact_created: bool,
+        message: CheckArtifactCreate | RequestArtifactGenerate,
     ) -> ArtifactGenerateResult:
         response = await self.sdk.query_gql_query(
             name=message.query,
@@ -1269,24 +1281,30 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
             branch_name=message.branch_name,
             timeout=message.timeout,
         )
+        branch = registry.get_branch_from_registry(branch=message.branch_name)
+
+        previous_checksum = artifact.checksum.value
+        previous_storage_id = artifact.storage_id.value
 
         if message.transform_type == InfrahubKind.TRANSFORMJINJA2:
-            artifact_content = await self.render_jinja2_template(
+            artifact_content = await self.render_jinja2_template.with_options(timeout_seconds=message.timeout)(
                 commit=message.commit, location=message.transform_location, data=response
-            )
+            )  # type: ignore[misc]
         elif message.transform_type == InfrahubKind.TRANSFORMPYTHON:
-            artifact_content = await self.execute_python_transform(
+            artifact_content = await self.execute_python_transform.with_options(timeout_seconds=message.timeout)(
                 branch_name=message.branch_name,
                 commit=message.commit,
                 location=message.transform_location,
                 data=response,
                 client=self.sdk,
-            )
+            )  # type: ignore[misc]
 
-        if message.content_type == "application/json":
+        if message.content_type == ContentType.APPLICATION_JSON.value and isinstance(artifact_content, dict):
             artifact_content_str = ujson.dumps(artifact_content, indent=2)
-        elif message.content_type == "text/plain":
-            artifact_content_str = artifact_content
+        elif message.content_type == ContentType.APPLICATION_YAML.value and isinstance(artifact_content, dict):
+            artifact_content_str = yaml.dump(artifact_content, indent=2)
+        else:
+            artifact_content_str = str(artifact_content)
 
         checksum = hashlib.md5(bytes(artifact_content_str, encoding="utf-8"), usedforsecurity=False).hexdigest()
 
@@ -1301,6 +1319,24 @@ class InfrahubRepositoryIntegrator(InfrahubRepositoryBase):  # pylint: disable=t
         artifact.content_type.value = message.content_type
         artifact.checksum.value = checksum
         artifact.storage_id.value = storage_id
-        artifact.status.value = "Ready"
+        artifact.status.value = ArtifactStatus.READY.value
+        if artifact.name.value != message.artifact_name:
+            artifact.name.value = message.artifact_name
         await artifact.save()
+
+        event_class = ArtifactCreatedEvent if artifact_created else ArtifactUpdatedEvent
+
+        event = event_class(
+            node_id=artifact.id,
+            target_id=message.target_id,
+            target_kind=message.target_kind,
+            artifact_definition_id=message.artifact_definition,
+            meta=EventMeta.from_context(context=message.context, branch=branch),
+            checksum=checksum,
+            checksum_previous=previous_checksum,
+            storage_id=storage_id,
+            storage_id_previous=previous_storage_id,
+        )
+
+        await self.service.event.send(event=event)
         return ArtifactGenerateResult(changed=True, checksum=checksum, storage_id=storage_id, artifact_id=artifact.id)
