@@ -1,5 +1,6 @@
 from pydantic import Field
 
+from infrahub.context import InfrahubContext
 from infrahub.message_bus import InfrahubMessage
 
 
@@ -8,3 +9,5 @@ class EventBranchMerge(InfrahubMessage):
 
     source_branch: str = Field(..., description="The source branch")
     target_branch: str = Field(..., description="The target branch")
+
+    context: InfrahubContext = Field(..., description="The context of the event")
