@@ -22,10 +22,10 @@ if TYPE_CHECKING:
     from typing_extensions import Self
 
     from .client import InfrahubClient, InfrahubClientSync
+    from .context import RequestContext
     from .schema import AttributeSchemaAPI, MainSchemaTypesAPI, RelationshipSchemaAPI
     from .types import Order
 
-# pylint: disable=too-many-lines
 
 PROPERTIES_FLAG = ["is_visible", "is_protected"]
 PROPERTIES_OBJECT = ["source", "owner"]
@@ -188,6 +188,7 @@ class RelatedNodeBase:
             if node_data:
                 self._id = node_data.get("id", None)
                 self._hfid = node_data.get("hfid", None)
+                self._kind = node_data.get("kind", None)
                 self._display_label = node_data.get("display_label", None)
                 self._typename = node_data.get("__typename", None)
 
@@ -256,6 +257,8 @@ class RelatedNodeBase:
             data["id"] = self.id
         elif self.hfid is not None:
             data["hfid"] = self.hfid
+            if self._kind is not None:
+                data["kind"] = self._kind
 
         for prop_name in self._properties:
             if getattr(self, prop_name) is not None:
@@ -767,6 +770,16 @@ class InfrahubNodeBase:
                 Attribute(name=attr_name, schema=attr_schema, data=attr_data),
             )
 
+    def _get_request_context(self, request_context: RequestContext | None = None) -> dict[str, Any] | None:
+        if request_context:
+            return request_context.model_dump(exclude_none=True)
+
+        client: InfrahubClient | InfrahubClientSync | None = getattr(self, "_client", None)
+        if not client or not client.request_context:
+            return None
+
+        return client.request_context.model_dump(exclude_none=True)
+
     def _init_relationships(self, data: dict | None = None) -> None:
         pass
 
@@ -795,13 +808,18 @@ class InfrahubNodeBase:
     def get_raw_graphql_data(self) -> dict | None:
         return self._data
 
-    def _generate_input_data(self, exclude_unmodified: bool = False, exclude_hfid: bool = False) -> dict[str, dict]:  # noqa: C901
+    def _generate_input_data(  # noqa: C901
+        self,
+        exclude_unmodified: bool = False,
+        exclude_hfid: bool = False,
+        request_context: RequestContext | None = None,
+    ) -> dict[str, dict]:
         """Generate a dictionary that represent the input data required by a mutation.
 
         Returns:
             dict[str, Dict]: Representation of an input data in dict format
         """
-        # pylint: disable=too-many-branches
+
         data = {}
         variables = {}
 
@@ -873,7 +891,15 @@ class InfrahubNodeBase:
         elif self.hfid is not None and not exclude_hfid:
             data["hfid"] = self.hfid
 
-        return {"data": {"data": data}, "variables": variables, "mutation_variables": mutation_variables}
+        mutation_payload = {"data": data}
+        if context_data := self._get_request_context(request_context=request_context):
+            mutation_payload["context"] = context_data
+
+        return {
+            "data": mutation_payload,
+            "variables": variables,
+            "mutation_variables": mutation_variables,
+        }
 
     @staticmethod
     def _strip_unmodified_dict(data: dict, original_data: dict, variables: dict, item: str) -> None:
@@ -1074,7 +1100,7 @@ class InfrahubNode(InfrahubNodeBase):
         timeout: int | None = None,
     ) -> Self:
         if not schema:
-            node_kind = data.get("__typename", None) or data.get("node", {}).get("__typename", None)
+            node_kind = data.get("__typename") or data.get("node", {}).get("__typename", None)
             if not node_kind:
                 raise ValueError("Unable to determine the type of the node, __typename not present in data")
             schema = await client.schema.get(kind=node_kind, branch=branch, timeout=timeout)
@@ -1119,19 +1145,22 @@ class InfrahubNode(InfrahubNodeBase):
     async def artifact_generate(self, name: str) -> None:
         self._validate_artifact_support(ARTIFACT_GENERATE_FEATURE_NOT_SUPPORTED_MESSAGE)
 
-        artifact = await self._client.get(kind="CoreArtifact", definition__name__value=name, object__ids=[self.id])
+        artifact = await self._client.get(kind="CoreArtifact", name__value=name, object__ids=[self.id])
         await artifact.definition.fetch()  # type: ignore[attr-defined]
         await artifact.definition.peer.generate([artifact.id])  # type: ignore[attr-defined]
 
     async def artifact_fetch(self, name: str) -> str | dict[str, Any]:
         self._validate_artifact_support(ARTIFACT_GENERATE_FEATURE_NOT_SUPPORTED_MESSAGE)
 
-        artifact = await self._client.get(kind="CoreArtifact", definition__name__value=name, object__ids=[self.id])
+        artifact = await self._client.get(kind="CoreArtifact", name__value=name, object__ids=[self.id])
         content = await self._client.object_store.get(identifier=artifact.storage_id.value)  # type: ignore[attr-defined]
         return content
 
-    async def delete(self, timeout: int | None = None) -> None:
+    async def delete(self, timeout: int | None = None, request_context: RequestContext | None = None) -> None:
         input_data = {"data": {"id": self.id}}
+        if context_data := self._get_request_context(request_context=request_context):
+            input_data["context"] = context_data
+
         mutation_query = {"ok": None}
         query = Mutation(
             mutation=f"{self._schema.kind}Delete",
@@ -1146,12 +1175,16 @@ class InfrahubNode(InfrahubNodeBase):
         )
 
     async def save(
-        self, allow_upsert: bool = False, update_group_context: bool | None = None, timeout: int | None = None
+        self,
+        allow_upsert: bool = False,
+        update_group_context: bool | None = None,
+        timeout: int | None = None,
+        request_context: RequestContext | None = None,
     ) -> None:
         if self._existing is False or allow_upsert is True:
-            await self.create(allow_upsert=allow_upsert, timeout=timeout)
+            await self.create(allow_upsert=allow_upsert, timeout=timeout, request_context=request_context)
         else:
-            await self.update(timeout=timeout)
+            await self.update(timeout=timeout, request_context=request_context)
 
         if update_group_context is None and self._client.mode == InfrahubClientMode.TRACKING:
             update_group_context = True
@@ -1251,7 +1284,6 @@ class InfrahubNode(InfrahubNodeBase):
         Returns:
             dict[str, Union[Any, Dict]]: GraphQL query in dictionary format
         """
-        # pylint: disable=too-many-branches
 
         data: dict[str, Any] = {}
 
@@ -1381,15 +1413,17 @@ class InfrahubNode(InfrahubNodeBase):
             await related_node.fetch(timeout=timeout)
             setattr(self, rel_name, related_node)
 
-    async def create(self, allow_upsert: bool = False, timeout: int | None = None) -> None:
+    async def create(
+        self, allow_upsert: bool = False, timeout: int | None = None, request_context: RequestContext | None = None
+    ) -> None:
         mutation_query = self._generate_mutation_query()
 
         if allow_upsert:
-            input_data = self._generate_input_data(exclude_hfid=False)
+            input_data = self._generate_input_data(exclude_hfid=False, request_context=request_context)
             mutation_name = f"{self._schema.kind}Upsert"
             tracker = f"mutation-{str(self._schema.kind).lower()}-upsert"
         else:
-            input_data = self._generate_input_data(exclude_hfid=True)
+            input_data = self._generate_input_data(exclude_hfid=True, request_context=request_context)
             mutation_name = f"{self._schema.kind}Create"
             tracker = f"mutation-{str(self._schema.kind).lower()}-create"
         query = Mutation(
@@ -1407,8 +1441,10 @@ class InfrahubNode(InfrahubNodeBase):
         )
         await self._process_mutation_result(mutation_name=mutation_name, response=response, timeout=timeout)
 
-    async def update(self, do_full_update: bool = False, timeout: int | None = None) -> None:
-        input_data = self._generate_input_data(exclude_unmodified=not do_full_update)
+    async def update(
+        self, do_full_update: bool = False, timeout: int | None = None, request_context: RequestContext | None = None
+    ) -> None:
+        input_data = self._generate_input_data(exclude_unmodified=not do_full_update, request_context=request_context)
         mutation_query = self._generate_mutation_query()
         mutation_name = f"{self._schema.kind}Update"
 
@@ -1594,7 +1630,7 @@ class InfrahubNodeSync(InfrahubNodeBase):
         timeout: int | None = None,
     ) -> Self:
         if not schema:
-            node_kind = data.get("__typename", None) or data.get("node", {}).get("__typename", None)
+            node_kind = data.get("__typename") or data.get("node", {}).get("__typename", None)
             if not node_kind:
                 raise ValueError("Unable to determine the type of the node, __typename not present in data")
             schema = client.schema.get(kind=node_kind, branch=branch, timeout=timeout)
@@ -1637,18 +1673,21 @@ class InfrahubNodeSync(InfrahubNodeBase):
 
     def artifact_generate(self, name: str) -> None:
         self._validate_artifact_support(ARTIFACT_GENERATE_FEATURE_NOT_SUPPORTED_MESSAGE)
-        artifact = self._client.get(kind="CoreArtifact", definition__name__value=name, object__ids=[self.id])
+        artifact = self._client.get(kind="CoreArtifact", name__value=name, object__ids=[self.id])
         artifact.definition.fetch()  # type: ignore[attr-defined]
         artifact.definition.peer.generate([artifact.id])  # type: ignore[attr-defined]
 
     def artifact_fetch(self, name: str) -> str | dict[str, Any]:
         self._validate_artifact_support(ARTIFACT_FETCH_FEATURE_NOT_SUPPORTED_MESSAGE)
-        artifact = self._client.get(kind="CoreArtifact", definition__name__value=name, object__ids=[self.id])
+        artifact = self._client.get(kind="CoreArtifact", name__value=name, object__ids=[self.id])
         content = self._client.object_store.get(identifier=artifact.storage_id.value)  # type: ignore[attr-defined]
         return content
 
-    def delete(self, timeout: int | None = None) -> None:
+    def delete(self, timeout: int | None = None, request_context: RequestContext | None = None) -> None:
         input_data = {"data": {"id": self.id}}
+        if context_data := self._get_request_context(request_context=request_context):
+            input_data["context"] = context_data
+
         mutation_query = {"ok": None}
         query = Mutation(
             mutation=f"{self._schema.kind}Delete",
@@ -1663,12 +1702,16 @@ class InfrahubNodeSync(InfrahubNodeBase):
         )
 
     def save(
-        self, allow_upsert: bool = False, update_group_context: bool | None = None, timeout: int | None = None
+        self,
+        allow_upsert: bool = False,
+        update_group_context: bool | None = None,
+        timeout: int | None = None,
+        request_context: RequestContext | None = None,
     ) -> None:
         if self._existing is False or allow_upsert is True:
-            self.create(allow_upsert=allow_upsert, timeout=timeout)
+            self.create(allow_upsert=allow_upsert, timeout=timeout, request_context=request_context)
         else:
-            self.update(timeout=timeout)
+            self.update(timeout=timeout, request_context=request_context)
 
         if update_group_context is None and self._client.mode == InfrahubClientMode.TRACKING:
             update_group_context = True
@@ -1763,7 +1806,6 @@ class InfrahubNodeSync(InfrahubNodeBase):
         Returns:
             dict[str, Union[Any, Dict]]: GraphQL query in dictionary format
         """
-        # pylint: disable=too-many-branches
 
         data: dict[str, Any] = {}
 
@@ -1893,15 +1935,17 @@ class InfrahubNodeSync(InfrahubNodeBase):
             related_node.fetch(timeout=timeout)
             setattr(self, rel_name, related_node)
 
-    def create(self, allow_upsert: bool = False, timeout: int | None = None) -> None:
+    def create(
+        self, allow_upsert: bool = False, timeout: int | None = None, request_context: RequestContext | None = None
+    ) -> None:
         mutation_query = self._generate_mutation_query()
 
         if allow_upsert:
-            input_data = self._generate_input_data(exclude_hfid=False)
+            input_data = self._generate_input_data(exclude_hfid=False, request_context=request_context)
             mutation_name = f"{self._schema.kind}Upsert"
             tracker = f"mutation-{str(self._schema.kind).lower()}-upsert"
         else:
-            input_data = self._generate_input_data(exclude_hfid=True)
+            input_data = self._generate_input_data(exclude_hfid=True, request_context=request_context)
             mutation_name = f"{self._schema.kind}Create"
             tracker = f"mutation-{str(self._schema.kind).lower()}-create"
         query = Mutation(
@@ -1920,8 +1964,10 @@ class InfrahubNodeSync(InfrahubNodeBase):
         )
         self._process_mutation_result(mutation_name=mutation_name, response=response, timeout=timeout)
 
-    def update(self, do_full_update: bool = False, timeout: int | None = None) -> None:
-        input_data = self._generate_input_data(exclude_unmodified=not do_full_update)
+    def update(
+        self, do_full_update: bool = False, timeout: int | None = None, request_context: RequestContext | None = None
+    ) -> None:
+        input_data = self._generate_input_data(exclude_unmodified=not do_full_update, request_context=request_context)
         mutation_query = self._generate_mutation_query()
         mutation_name = f"{self._schema.kind}Update"
 
