@@ -34,6 +34,7 @@ from .main import (
     RelationshipSchemaAPI,
     SchemaRoot,
     SchemaRootAPI,
+    TemplateSchemaAPI,
 )
 
 if TYPE_CHECKING:
@@ -58,12 +59,10 @@ __all__ = [
     "RelationshipSchemaAPI",
     "SchemaRoot",
     "SchemaRootAPI",
+    "TemplateSchemaAPI",
 ]
 
 
-# pylint: disable=redefined-builtin
-
-
 class DropdownMutationOptionalArgs(TypedDict):
     color: str | None
     description: str | None
@@ -81,8 +80,10 @@ class EnumMutation(str, Enum):
 
 
 MainSchemaTypes: TypeAlias = Union[NodeSchema, GenericSchema]
-MainSchemaTypesAPI: TypeAlias = Union[NodeSchemaAPI, GenericSchemaAPI, ProfileSchemaAPI]
-MainSchemaTypesAll: TypeAlias = Union[NodeSchema, GenericSchema, NodeSchemaAPI, GenericSchemaAPI, ProfileSchemaAPI]
+MainSchemaTypesAPI: TypeAlias = Union[NodeSchemaAPI, GenericSchemaAPI, ProfileSchemaAPI, TemplateSchemaAPI]
+MainSchemaTypesAll: TypeAlias = Union[
+    NodeSchema, GenericSchema, NodeSchemaAPI, GenericSchemaAPI, ProfileSchemaAPI, TemplateSchemaAPI
+]
 
 
 class InfrahubSchemaBase:
@@ -420,6 +421,10 @@ class InfrahubSchema(InfrahubSchemaBase):
             profile = ProfileSchemaAPI(**profile_schema)
             nodes[profile.kind] = profile
 
+        for template_schema in data.get("templates", []):
+            template = TemplateSchemaAPI(**template_schema)
+            nodes[template.kind] = template
+
         return nodes
 
 
@@ -624,6 +629,10 @@ class InfrahubSchemaSync(InfrahubSchemaBase):
             profile = ProfileSchemaAPI(**profile_schema)
             nodes[profile.kind] = profile
 
+        for template_schema in data.get("templates", []):
+            template = TemplateSchemaAPI(**template_schema)
+            nodes[template.kind] = template
+
         return nodes
 
     def load(
